@@ -306,7 +306,7 @@ async fn restore_command(target: String, pattern: Option<String>) -> Result<(),
 /// here:
 pub struct Shell {
     /// Readline instance handling input and callbacks
-    rl: rustyline::Editor<CliHelper>,
+    rl: rustyline::Editor<CliHelper, rustyline::history::MemHistory>,
 
     /// Interactive prompt.
     prompt: String,
@@ -352,7 +352,10 @@ impl Shell {
         archive: Accessor,
     ) -> Result<Self, Error> {
         let cli_helper = CliHelper::new(catalog_shell_cli());
-        let mut rl = rustyline::Editor::<CliHelper>::new();
+        let mut rl = rustyline::Editor::<CliHelper, _>::with_history(
+            rustyline::Config::default(),
+            rustyline::history::MemHistory::new(),
+        )?;
         rl.set_helper(Some(cli_helper));
 
         let mut position = Vec::new();
@@ -426,7 +429,7 @@ impl Shell {
             let _ =
                 cli::handle_command_future(helper.cmd_def(), "", args, cli::CliEnvironment::new())
                     .await;
-            this.rl.add_history_entry(line);
+            let _ = this.rl.add_history_entry(line);
             this.update_prompt();
         }
         Ok(())
@@ -460,7 +463,7 @@ impl Shell {
         )
         .await?;
         let path = Self::format_path_stack(&stack);
-        println!("{:?}", path);
+        println!("{path:?}");
         Ok(())
     }
 
@@ -859,9 +862,9 @@ impl Shell {
         let path = Self::format_path_stack(&stack);
         let entry = MatchEntry::include(MatchPattern::Literal(path.as_bytes().to_vec()));
         if self.selected.insert(path.clone(), entry).is_some() {
-            println!("path already selected: {:?}", path);
+            println!("path already selected: {path:?}");
         } else {
-            println!("added path: {:?}", path);
+            println!("added path: {path:?}");
         }
 
         Ok(())
@@ -880,9 +883,9 @@ impl Shell {
         let path = Self::format_path_stack(&stack);
 
         if self.selected.remove(&path).is_some() {
-            println!("removed path from selection: {:?}", path);
+            println!("removed path from selection: {path:?}");
         } else {
-            println!("path not selected: {:?}", path);
+            println!("path not selected: {path:?}");
         }
 
         Ok(())
@@ -904,7 +907,7 @@ impl Shell {
 
     async fn list_selected_patterns(&self) -> Result<(), Error> {
         for entry in self.selected.keys() {
-            println!("{:?}", entry);
+            println!("{entry:?}");
         }
         Ok(())
     }
