@@ -26,7 +26,7 @@ use proxmox_log::{debug, error, info};
 use proxmox_sys::c_result;
 use proxmox_sys::fs::{create_path, CreateOptions};
 
-use proxmox_compression::zip::{ZipEncoder, ZipEntry};
+use proxmox_compression::zip::{FileType, ZipEncoder, ZipEntry};
 
 use crate::pxar::dir_stack::PxarDirStack;
 use crate::pxar::metadata;
@@ -627,7 +627,7 @@ impl Extractor {
                 target.as_c_str(),
                 Some(parent),
                 file_name,
-                nix::unistd::LinkatFlags::NoSymlinkFollow,
+                nix::fcntl::AtFlags::empty(),
             )
         };
 
@@ -697,8 +697,13 @@ impl Extractor {
         }
         let mut file = unsafe {
             std::fs::File::from_raw_fd(
-                nix::fcntl::openat(parent, file_name, oflags, Mode::from_bits(0o600).unwrap())
-                    .with_context(|| format!("failed to create file {file_name:?}"))?,
+                nix::fcntl::openat(
+                    Some(parent),
+                    file_name,
+                    oflags,
+                    Mode::from_bits(0o600).unwrap(),
+                )
+                .with_context(|| format!("failed to create file {file_name:?}"))?,
             )
         };
 
@@ -722,7 +727,7 @@ impl Extractor {
         }
 
         if result.seeked_last {
-            while match nix::unistd::ftruncate(file.as_raw_fd(), size as i64) {
+            while match nix::unistd::ftruncate(&file, size as i64) {
                 Ok(_) => false,
                 Err(nix::errno::Errno::EINTR) => true,
                 Err(err) => return Err(err).context("error setting file size"),
@@ -758,8 +763,13 @@ impl Extractor {
         }
         let mut file = tokio::fs::File::from_std(unsafe {
             std::fs::File::from_raw_fd(
-                nix::fcntl::openat(parent, file_name, oflags, Mode::from_bits(0o600).unwrap())
-                    .with_context(|| format!("failed to create file {file_name:?}"))?,
+                nix::fcntl::openat(
+                    Some(parent),
+                    file_name,
+                    oflags,
+                    Mode::from_bits(0o600).unwrap(),
+                )
+                .with_context(|| format!("failed to create file {file_name:?}"))?,
             )
         });
 
@@ -784,7 +794,7 @@ impl Extractor {
         }
 
         if result.seeked_last {
-            while match nix::unistd::ftruncate(file.as_raw_fd(), size as i64) {
+            while match nix::unistd::ftruncate(&file, size as i64) {
                 Ok(_) => false,
                 Err(nix::errno::Errno::EINTR) => true,
                 Err(err) => return Err(err).context("error setting file size"),
@@ -1024,7 +1034,7 @@ where
                 path,
                 metadata.stat.mtime.secs,
                 metadata.stat.mode as u16,
-                false,
+                FileType::Directory,
             );
             zip.add_entry::<FileContents<T>>(entry, None).await?;
         }
@@ -1043,7 +1053,7 @@ where
                         path,
                         metadata.stat.mtime.secs,
                         metadata.stat.mode as u16,
-                        true,
+                        FileType::Regular,
                     );
                     let contents = decoder.contents().await?;
                     zip.add_entry(entry, contents)
@@ -1054,7 +1064,7 @@ where
                     let entry = root
                         .lookup(&path)
                         .await?
-                        .with_context(|| format!("error looking up {:?}", path))?;
+                        .with_context(|| format!("error looking up {path:?}"))?;
                     let realfile = accessor.follow_hardlink(&entry).await?;
                     let metadata = realfile.entry().metadata();
                     debug!("adding '{}' to zip", path.display());
@@ -1062,7 +1072,7 @@ where
                         path,
                         metadata.stat.mtime.secs,
                         metadata.stat.mode as u16,
-                        true,
+                        FileType::Regular,
                     );
                     let contents = decoder.contents().await?;
                     zip.add_entry(entry, contents)
@@ -1075,17 +1085,28 @@ where
                         path,
                         metadata.stat.mtime.secs,
                         metadata.stat.mode as u16,
-                        false,
+                        FileType::Directory,
                     );
                     zip.add_entry::<FileContents<T>>(entry, None).await?;
                 }
+                EntryKind::Symlink(target) => {
+                    debug!("adding '{}' to zip", path.display());
+                    let entry = ZipEntry::new(
+                        path,
+                        metadata.stat.mtime.secs,
+                        metadata.stat.mode as u16,
+                        FileType::Symlink,
+                    );
+                    let target = io::Cursor::<&[u8]>::new(target.as_ref());
+                    zip.add_entry(entry, Some(target)).await?;
+                }
                 _ => {} // ignore all else
             };
         }
     }
 
     zip.finish().await.map_err(|err| {
-        eprintln!("error during finishing of zip: {}", err);
+        eprintln!("error during finishing of zip: {err}");
         err
     })
 }
@@ -1312,16 +1333,11 @@ where
         }
         .await
         {
-            let display_string = entry.path().display().to_string();
-            error!(
-                "error extracting {}: {}",
-                if matches!(entry.kind(), EntryKind::GoodbyeTable) {
-                    "<directory>"
-                } else {
-                    &display_string
-                },
-                err
-            );
+            let path_display = match entry.kind() {
+                EntryKind::GoodbyeTable => "<directory>".to_string(),
+                _ => entry.path().display().to_string(),
+            };
+            error!("error extracting {path_display}: {err}");
         }
 
         if dir_level < 0 {
