@@ -127,6 +127,12 @@ pub fn check_acl_path(path: &str) -> Result<(), Error> {
                         _ => {}
                     }
                 }
+                "s3-endpoint" => {
+                    // /system/s3-endpoint/{id}
+                    if components_len <= 3 {
+                        return Ok(());
+                    }
+                }
                 _ => {}
             }
         }
@@ -443,7 +449,7 @@ impl AclTree {
 
         for (auth_id, roles) in &node.users {
             // no need to save, because root is always 'Administrator'
-            if !auth_id.is_token() && auth_id.user() == "root@pam" {
+            if !auth_id.is_token() && auth_id.user() == Userid::root_userid() {
                 continue;
             }
             for (role, propagate) in roles {
@@ -459,7 +465,7 @@ impl AclTree {
 
         for (group, roles) in &node.groups {
             for (role, propagate) in roles {
-                let group = format!("@{}", group);
+                let group = format!("@{group}");
                 if *propagate {
                     role_ug_map1.entry(role).or_default().insert(group);
                 } else {
@@ -527,7 +533,7 @@ impl AclTree {
         }
 
         for (name, child) in node.children.iter() {
-            let child_path = format!("{}/{}", path, name);
+            let child_path = format!("{path}/{name}");
             Self::write_node_config(child, &child_path, w)?;
         }
 
@@ -783,8 +789,7 @@ mod test {
 
         assert_eq!(
             roles, expected_roles,
-            "\nat check_roles for '{}' on '{}'",
-            auth_id, path
+            "\nat check_roles for '{auth_id}' on '{path}'"
         );
     }
 
