@@ -203,7 +203,7 @@ impl DirEntry {
 
     /// Check if DirEntry is a symlink
     pub fn is_symlink(&self) -> bool {
-        matches!(self.attr, DirEntryAttribute::Symlink { .. })
+        matches!(self.attr, DirEntryAttribute::Symlink)
     }
 }
 
@@ -973,8 +973,8 @@ impl ArchiveEntry {
         size: Option<u64>,
     ) -> Self {
         Self {
-            filepath: base64::encode(filepath),
-            text: String::from_utf8_lossy(filepath.split(|x| *x == b'/').last().unwrap())
+            filepath: proxmox_base64::encode(filepath),
+            text: String::from_utf8_lossy(filepath.split(|x| *x == b'/').next_back().unwrap())
                 .to_string(),
             entry_type: match entry_type {
                 Some(entry_type) => CatalogEntryType::from(entry_type).to_string(),
