@@ -28,17 +28,17 @@ use pbs_tape::{
 
 pub const FILE_MARK_COUNT_SCHEMA: Schema = IntegerSchema::new("File mark count.")
     .minimum(1)
-    .maximum(i32::MAX as isize)
+    .maximum(i32::MAX as i64)
     .schema();
 
 pub const FILE_MARK_POSITION_SCHEMA: Schema = IntegerSchema::new("File mark position (0 is BOT).")
     .minimum(0)
-    .maximum(i32::MAX as isize)
+    .maximum(i32::MAX as i64)
     .schema();
 
 pub const RECORD_COUNT_SCHEMA: Schema = IntegerSchema::new("Record count.")
     .minimum(1)
-    .maximum(i32::MAX as isize)
+    .maximum(i32::MAX as i64)
     .schema();
 
 pub const DRIVE_OPTION_SCHEMA: Schema =
@@ -278,9 +278,7 @@ fn tape_alert_flags(param: Value) -> Result<(), Error> {
     let output_format = get_output_format(&param);
 
     let mut handle = get_tape_handle(&param)?;
-    let result = handle
-        .tape_alert_flags()
-        .map(|flags| format!("{:?}", flags));
+    let result = handle.tape_alert_flags().map(|flags| format!("{flags:?}"));
 
     if output_format == "json-pretty" {
         let result = result.map_err(|err: Error| err.to_string());
@@ -299,7 +297,7 @@ fn tape_alert_flags(param: Value) -> Result<(), Error> {
     }
 
     let flags = result?;
-    println!("Tape Alert Flags: {}", flags);
+    println!("Tape Alert Flags: {flags}");
 
     Ok(())
 }
@@ -842,7 +840,7 @@ fn main() -> Result<(), Error> {
         .insert("weof", std_cmd(&API_METHOD_WEOF).arg_param(&["count"]));
 
     let mut rpcenv = CliEnvironment::new();
-    rpcenv.set_auth_id(Some(format!("{}@pam", username)));
+    rpcenv.set_auth_id(Some(format!("{username}@pam")));
 
     run_cli_command(cmd_def, rpcenv, None);
 
