@@ -32,10 +32,10 @@ impl std::fmt::Display for SenseInfo {
             .unwrap_or_else(|| format!("Invalid sense {:02X}", self.sense_key));
 
         if self.asc == 0 && self.ascq == 0 {
-            write!(f, "{}", sense_text)
+            write!(f, "{sense_text}")
         } else {
             let additional_sense_text = get_asc_ascq_string(self.asc, self.ascq);
-            write!(f, "{}, {}", sense_text, additional_sense_text)
+            write!(f, "{sense_text}, {additional_sense_text}")
         }
     }
 }
@@ -429,7 +429,7 @@ pub fn get_asc_ascq_string(asc: u8, ascq: u8) -> String {
         }
         Ok(unsafe { CStr::from_ptr(res) }.to_str()?.to_owned())
     })
-    .unwrap_or_else(|_err: Error| format!("ASC={:02x}x, ASCQ={:02x}x", asc, ascq))
+    .unwrap_or_else(|_err: Error| format!("ASC={asc:02x}x, ASCQ={ascq:02x}x"))
 }
 
 /// Allocate a page aligned buffer
@@ -512,7 +512,7 @@ impl<'a, F: AsRawFd> SgRaw<'a, F> {
             SCSI_PT_DO_TIMEOUT => return Err(format_err!("do_scsi_pt failed - timeout").into()),
             code if code < 0 => {
                 let errno = unsafe { get_scsi_pt_os_err(ptvp.as_ptr()) };
-                let err = nix::errno::Errno::from_i32(errno);
+                let err = nix::errno::Errno::from_raw(errno);
                 return Err(format_err!("do_scsi_pt failed with err {}", err).into());
             }
             unknown => {
@@ -594,7 +594,7 @@ impl<'a, F: AsRawFd> SgRaw<'a, F> {
             }
             SCSI_PT_RESULT_OS_ERR => {
                 let errno = unsafe { get_scsi_pt_os_err(ptvp.as_ptr()) };
-                let err = nix::errno::Errno::from_i32(errno);
+                let err = nix::errno::Errno::from_raw(errno);
                 Err(format_err!("scsi command failed with err {}", err).into())
             }
             unknown => {
