@@ -46,8 +46,8 @@ use pbs_client::tools::{
 use pbs_client::{
     delete_ticket_info, parse_backup_specification, view_task_result, BackupDetectionMode,
     BackupReader, BackupRepository, BackupSpecificationType, BackupStats, BackupWriter,
-    ChunkStream, FixedChunkStream, HttpClient, InjectionData, PxarBackupStream, RemoteChunkReader,
-    UploadOptions, BACKUP_SOURCE_SCHEMA,
+    BackupWriterOptions, ChunkStream, FixedChunkStream, HttpClient, InjectionData,
+    PxarBackupStream, RemoteChunkReader, UploadOptions, BACKUP_SOURCE_SCHEMA,
 };
 use pbs_datastore::catalog::{BackupCatalogWriter, CatalogReader, CatalogWriter};
 use pbs_datastore::chunk_store::verify_chunk_size;
@@ -145,7 +145,7 @@ async fn api_datastore_list_snapshots(
     ns: &BackupNamespace,
     group: Option<&BackupGroup>,
 ) -> Result<Value, Error> {
-    let path = format!("api2/json/admin/datastore/{}/snapshots", store);
+    let path = format!("api2/json/admin/datastore/{store}/snapshots");
 
     let mut args = match group {
         Some(group) => serde_json::to_value(group)?,
@@ -537,7 +537,7 @@ async fn api_version(param: Value) -> Result<(), Error> {
         if let Some(server) = version_info["server"].as_object() {
             let server_version = server["version"].as_str().unwrap();
             let server_release = server["release"].as_str().unwrap();
-            println!("server version: {}.{}", server_version, server_release);
+            println!("server version: {server_version}.{server_release}");
         }
     } else {
         format_and_print_result(&version_info, &output_format);
@@ -726,7 +726,7 @@ fn spawn_catalog_upload(
                 type: Integer,
                 description: "Max number of entries to hold in memory.",
                 optional: true,
-                default: pbs_client::pxar::ENCODER_MAX_ENTRIES as isize,
+                default: pbs_client::pxar::ENCODER_MAX_ENTRIES as i64,
             },
             "dry-run": {
                 type: Boolean,
@@ -742,6 +742,12 @@ fn spawn_catalog_upload(
                 optional: true,
                 default: false,
             },
+            "no-cache": {
+                type: Boolean,
+                description: "Bypass local datastore cache for network storages.",
+                optional: true,
+                default: false,
+            },
         }
     }
  )]
@@ -754,6 +760,7 @@ async fn create_backup(
     change_detection_mode: Option<BackupDetectionMode>,
     dry_run: bool,
     skip_e2big_xattr: bool,
+    no_cache: bool,
     limit: ClientRateLimitConfig,
     _info: &ApiMethod,
     _rpcenv: &mut dyn RpcEnvironment,
@@ -954,12 +961,15 @@ async fn create_backup(
 
     let client = BackupWriter::start(
         &http_client,
-        crypt_config.clone(),
-        repo.store(),
-        &backup_ns,
-        &snapshot,
-        true,
-        false,
+        BackupWriterOptions {
+            datastore: repo.store(),
+            ns: &backup_ns,
+            backup: &snapshot,
+            crypt_config: crypt_config.clone(),
+            debug: true,
+            benchmark: false,
+            no_cache,
+        },
     )
     .await?;
 
@@ -1907,8 +1917,8 @@ async fn status(param: Value) -> Result<Value, Error> {
         let total = record["total"].as_u64().unwrap();
         let roundup = total / 200;
         if let Some(per) = ((v + roundup) * 100).checked_div(total) {
-            let info = format!(" ({} %)", per);
-            Ok(format!("{} {:>8}", v, info))
+            let info = format!(" ({per} %)");
+            Ok(format!("{v} {info:>8}"))
         } else {
             bail!("Cannot render total percentage: denominator is zero");
         }
