@@ -5,11 +5,13 @@ use futures::*;
 use hex::FromHex;
 use hyper::header::{HeaderValue, CONNECTION, UPGRADE};
 use hyper::http::request::Parts;
-use hyper::{Body, Request, Response, StatusCode};
+use hyper::{body::Incoming, Request, Response, StatusCode};
+use hyper_util::service::TowerToHyperService;
 use serde::Deserialize;
 use serde_json::{json, Value};
 use tracing::warn;
 
+use proxmox_http::Body;
 use proxmox_rest_server::{H2Service, WorkerTask};
 use proxmox_router::{http_err, list_subdirs_api_method};
 use proxmox_router::{
@@ -51,6 +53,7 @@ pub const API_METHOD_UPGRADE_BACKUP: ApiMethod = ApiMethod::new(
             ("backup-time", false, &BACKUP_TIME_SCHEMA),
             ("debug", true, &BooleanSchema::new("Enable verbose debug logging.").schema()),
             ("benchmark", true, &BooleanSchema::new("Job is a benchmark (do not keep data).").schema()),
+            ("no-cache", true, &BooleanSchema::new("Disable local datastore cache for network storages").schema()),
         ]),
     )
 ).access(
@@ -69,7 +72,7 @@ pub(crate) fn optional_ns_param(param: &Value) -> Result<BackupNamespace, Error>
 
 fn upgrade_to_backup_protocol(
     parts: Parts,
-    req_body: Body,
+    req_body: Incoming,
     param: Value,
     _info: &ApiMethod,
     rpcenv: Box<dyn RpcEnvironment>,
@@ -77,6 +80,7 @@ fn upgrade_to_backup_protocol(
     async move {
         let debug = param["debug"].as_bool().unwrap_or(false);
         let benchmark = param["benchmark"].as_bool().unwrap_or(false);
+        let no_cache = param["no-cache"].as_bool().unwrap_or(false);
 
         let auth_id: Authid = rpcenv.get_auth_id().unwrap().parse()?;
 
@@ -140,7 +144,7 @@ fn upgrade_to_backup_protocol(
         };
 
         // lock backup group to only allow one backup per group at a time
-        let (owner, _group_guard) = datastore.create_locked_backup_group(
+        let (owner, group_guard) = datastore.create_locked_backup_group(
             backup_group.backup_ns(),
             backup_group.as_ref(),
             &auth_id,
@@ -166,7 +170,7 @@ fn upgrade_to_backup_protocol(
                     Ok(None) => {
                         // no verify state found, treat as valid
                         Some(info)
-                    },
+                    }
                     Err(err) => {
                         warn!("error parsing the snapshot manifest: {err:#}");
                         Some(info)
@@ -179,13 +183,14 @@ fn upgrade_to_backup_protocol(
 
         let backup_dir = backup_group.backup_dir(backup_dir_arg.time)?;
 
-        let _last_guard = if let Some(last) = &last_backup {
+        let last_guard = if let Some(last) = &last_backup {
             if backup_dir.backup_time() <= last.backup_dir.backup_time() {
                 bail!("backup timestamp is older than last backup.");
             }
 
             // lock last snapshot to prevent forgetting/pruning it during backup
-            let guard = last.backup_dir
+            let guard = last
+                .backup_dir
                 .lock_shared()
                 .with_context(|| format!("while locking last snapshot during backup '{last:?}'"))?;
             Some(guard)
@@ -204,14 +209,24 @@ fn upgrade_to_backup_protocol(
             Some(worker_id),
             auth_id.to_string(),
             true,
-            move |worker| {
+            move |worker| async move {
+                // Keep flock for the backup runtime by storing guards in backup env shared state.
+                // Drop them on successful backup finish or when dropping the env after cleanup in
+                // case of errors. The former is required for immediate subsequent backups (e.g.
+                // during a push sync) to be able to lock the group and snapshots.
+                let chunk_store_guard = datastore.try_shared_chunk_store_lock()?;
+                let backup_lock_guards =
+                    BackupLockGuards::new(last_guard, group_guard, snap_guard, chunk_store_guard);
+
                 let mut env = BackupEnvironment::new(
                     env_type,
                     auth_id,
                     worker.clone(),
                     datastore,
                     backup_dir,
-                );
+                    no_cache,
+                    backup_lock_guards,
+                )?;
 
                 env.debug = debug;
                 env.last_backup = last_backup;
@@ -236,7 +251,8 @@ fn upgrade_to_backup_protocol(
                     .and_then(move |conn| {
                         env2.debug("protocol upgrade done");
 
-                        let mut http = hyper::server::conn::http2::Builder::new(ExecInheritLogContext);
+                        let mut http =
+                            hyper::server::conn::http2::Builder::new(ExecInheritLogContext);
                         // increase window size: todo - find optiomal size
                         let window_size = 32 * 1024 * 1024; // max = (1 << 31) - 2
                         http.initial_stream_window_size(window_size);
@@ -244,74 +260,85 @@ fn upgrade_to_backup_protocol(
                         http.max_frame_size(4 * 1024 * 1024);
 
                         let env3 = env2.clone();
-                        http.serve_connection(conn, service).map(move |result| {
-                            match result {
-                                Err(err) => {
-                                    // Avoid  Transport endpoint is not connected (os error 107)
-                                    // fixme: find a better way to test for that error
-                                    if err.to_string().starts_with("connection error")
-                                        && env3.finished()
-                                    {
-                                        Ok(())
-                                    } else {
-                                        Err(Error::from(err))
+                        http.serve_connection(conn, TowerToHyperService::new(service))
+                            .map(move |result| {
+                                match result {
+                                    Err(err) => {
+                                        // Avoid  Transport endpoint is not connected (os error 107)
+                                        // fixme: find a better way to test for that error
+                                        if err.to_string().starts_with("connection error")
+                                            && env3.finished()
+                                        {
+                                            Ok(())
+                                        } else {
+                                            Err(Error::from(err))
+                                        }
                                     }
+                                    Ok(()) => Ok(()),
                                 }
-                                Ok(()) => Ok(()),
-                            }
-                        })
+                            })
                     });
                 let mut abort_future = abort_future.map(|_| Err(format_err!("task aborted")));
 
-                async move {
-                    // keep flock until task ends
-                    let _group_guard = _group_guard;
-                    let snap_guard = snap_guard;
-                    let _last_guard = _last_guard;
+                let res = select! {
+                    req = req_fut => req,
+                    abrt = abort_future => abrt,
+                };
+                if benchmark {
+                    env.log("benchmark finished successfully");
+                    proxmox_async::runtime::block_in_place(|| {
+                        env.datastore.remove_backup_dir(
+                            env.backup_dir.backup_ns(),
+                            env.backup_dir.as_ref(),
+                            true,
+                        )
+                    })?;
+                    return Ok(());
+                }
 
-                    let res = select! {
-                        req = req_fut => req,
-                        abrt = abort_future => abrt,
-                    };
-                    if benchmark {
-                        env.log("benchmark finished successfully");
-                        proxmox_async::runtime::block_in_place(|| env.remove_backup())?;
-                        return Ok(());
+                let verify = |env: BackupEnvironment| {
+                    if let Err(err) = env.verify_after_complete() {
+                        env.log(format!(
+                            "backup finished, but starting the requested verify task failed: {err}"
+                        ));
                     }
+                };
 
-                    let verify = |env: BackupEnvironment| {
-                        if let Err(err) = env.verify_after_complete(snap_guard) {
-                            env.log(format!(
-                                "backup finished, but starting the requested verify task failed: {}",
-                                err
-                            ));
-                        }
-                    };
-
-                    match (res, env.ensure_finished()) {
-                        (Ok(_), Ok(())) => {
-                            env.log("backup finished successfully");
-                            verify(env);
-                            Ok(())
-                        }
-                        (Err(err), Ok(())) => {
-                            // ignore errors after finish
-                            env.log(format!("backup had errors but finished: {}", err));
-                            verify(env);
-                            Ok(())
-                        }
-                        (Ok(_), Err(err)) => {
-                            env.log(format!("backup ended and finish failed: {}", err));
-                            env.log("removing unfinished backup");
-                            proxmox_async::runtime::block_in_place(|| env.remove_backup())?;
-                            Err(err)
-                        }
-                        (Err(err), Err(_)) => {
-                            env.log(format!("backup failed: {}", err));
-                            env.log("removing failed backup");
-                            proxmox_async::runtime::block_in_place(|| env.remove_backup())?;
-                            Err(err)
-                        }
+                match (res, env.ensure_finished()) {
+                    (Ok(_), Ok(())) => {
+                        env.log("backup finished successfully");
+                        verify(env);
+                        Ok(())
+                    }
+                    (Err(err), Ok(())) => {
+                        // ignore errors after finish
+                        env.log(format!("backup had errors but finished: {err}"));
+                        verify(env);
+                        Ok(())
+                    }
+                    (Ok(_), Err(err)) => {
+                        env.log(format!("backup ended and finish failed: {err}"));
+                        env.log("removing unfinished backup");
+                        proxmox_async::runtime::block_in_place(|| {
+                            env.datastore.remove_backup_dir(
+                                env.backup_dir.backup_ns(),
+                                env.backup_dir.as_ref(),
+                                true,
+                            )
+                        })?;
+                        Err(err)
+                    }
+                    (Err(err), Err(_)) => {
+                        env.log(format!("backup failed: {err}"));
+                        env.log("removing failed backup");
+                        proxmox_async::runtime::block_in_place(|| {
+                            env.datastore.remove_backup_dir(
+                                env.backup_dir.backup_ns(),
+                                env.backup_dir.as_ref(),
+                                true,
+                            )
+                        })?;
+                        Err(err)
                     }
                 }
             },
@@ -415,7 +442,7 @@ fn create_dynamic_index(
     let index = env.datastore.create_dynamic_writer(&path)?;
     let wid = env.register_dynamic_writer(index, name)?;
 
-    env.log(format!("created new dynamic index {} ({:?})", wid, path));
+    env.log(format!("created new dynamic index {wid} ({path:?})"));
 
     Ok(json!(wid))
 }
@@ -509,7 +536,7 @@ fn create_fixed_index(
 
     let wid = env.register_fixed_writer(writer, name, size, chunk_size as u32, incremental)?;
 
-    env.log(format!("created new fixed index {} ({:?})", wid, path));
+    env.log(format!("created new fixed index {wid} ({path:?})"));
 
     Ok(json!(wid))
 }
@@ -580,8 +607,7 @@ fn dynamic_append(
         env.dynamic_writer_append_chunk(wid, offset, size, &digest)?;
 
         env.debug(format!(
-            "successfully added chunk {} to dynamic index {} (offset {}, size {})",
-            digest_str, wid, offset, size
+            "successfully added chunk {digest_str} to dynamic index {wid} (offset {offset}, size {size})"
         ));
     }
 
@@ -654,8 +680,7 @@ fn fixed_append(
         env.fixed_writer_append_chunk(wid, offset, size, &digest)?;
 
         env.debug(format!(
-            "successfully added chunk {} to fixed index {} (offset {}, size {})",
-            digest_str, wid, offset, size
+            "successfully added chunk {digest_str} to fixed index {wid} (offset {offset}, size {size})"
         ));
     }
 
@@ -718,7 +743,7 @@ fn close_dynamic_index(
 
     env.dynamic_writer_close(wid, chunk_count, size, csum)?;
 
-    env.log(format!("successfully closed dynamic index {}", wid));
+    env.log(format!("successfully closed dynamic index {wid}"));
 
     Ok(Value::Null)
 }
@@ -771,7 +796,7 @@ fn close_fixed_index(
 
     env.fixed_writer_close(wid, chunk_count, size, csum)?;
 
-    env.log(format!("successfully closed fixed index {}", wid));
+    env.log(format!("successfully closed fixed index {wid}"));
 
     Ok(Value::Null)
 }
@@ -821,7 +846,7 @@ pub const API_METHOD_DOWNLOAD_PREVIOUS: ApiMethod = ApiMethod::new(
 
 fn download_previous(
     _parts: Parts,
-    _req_body: Body,
+    _req_body: Incoming,
     param: Value,
     _info: &ApiMethod,
     rpcenv: Box<dyn RpcEnvironment>,
