@@ -1,8 +1,10 @@
 use std::path::{Path, PathBuf};
+use std::sync::Arc;
 
 use ::serde::{Deserialize, Serialize};
-use anyhow::{bail, Context, Error};
+use anyhow::{bail, format_err, Context, Error};
 use hex::FromHex;
+use http_body_util::BodyExt;
 use serde_json::Value;
 use tracing::{info, warn};
 
@@ -12,30 +14,38 @@ use proxmox_section_config::SectionConfigData;
 use proxmox_uuid::Uuid;
 
 use pbs_api_types::{
-    Authid, DataStoreConfig, DataStoreConfigUpdater, DatastoreNotify, DatastoreTuning, KeepOptions,
-    MaintenanceMode, PruneJobConfig, PruneJobOptions, DATASTORE_SCHEMA, PRIV_DATASTORE_ALLOCATE,
-    PRIV_DATASTORE_AUDIT, PRIV_DATASTORE_MODIFY, PRIV_SYS_MODIFY, PROXMOX_CONFIG_DIGEST_SCHEMA,
-    UPID_SCHEMA,
+    Authid, DataStoreConfig, DataStoreConfigUpdater, DatastoreBackendType, DatastoreNotify,
+    DatastoreTuning, KeepOptions, MaintenanceMode, MaintenanceType, PruneJobConfig,
+    PruneJobOptions, DATASTORE_SCHEMA, PRIV_DATASTORE_ALLOCATE, PRIV_DATASTORE_AUDIT,
+    PRIV_DATASTORE_MODIFY, PRIV_SYS_MODIFY, PROXMOX_CONFIG_DIGEST_SCHEMA, UPID_SCHEMA,
 };
 use pbs_config::BackupLockGuard;
 use pbs_datastore::chunk_store::ChunkStore;
 
-use crate::api2::admin::{
-    datastore::do_mount_device, prune::list_prune_jobs, sync::list_config_sync_jobs,
-    verify::list_verification_jobs,
-};
+use crate::api2::admin::datastore::do_mount_device;
+use crate::api2::admin::prune::list_prune_jobs;
+use crate::api2::admin::sync::{list_config_sync_jobs, ListSyncDirection};
+use crate::api2::admin::verify::list_verification_jobs;
 use crate::api2::config::prune::{delete_prune_job, do_create_prune_job, has_prune_job};
 use crate::api2::config::sync::delete_sync_job;
 use crate::api2::config::tape_backup_job::{delete_tape_backup_job, list_tape_backup_jobs};
 use crate::api2::config::verify::delete_verification_job;
 use pbs_config::CachedUserInfo;
 
-use pbs_datastore::get_datastore_mount_status;
+use pbs_datastore::{get_datastore_mount_status, DataStore, S3_DATASTORE_IN_USE_MARKER};
 use proxmox_rest_server::WorkerTask;
+use proxmox_s3_client::{S3ObjectKey, S3_HTTP_REQUEST_TIMEOUT};
 
 use crate::server::jobstate;
 use crate::tools::disks::unmount_by_mountpoint;
 
+#[derive(Default, serde::Deserialize, serde::Serialize)]
+#[serde(rename_all = "kebab-case")]
+struct InUseContent {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    hostname: Option<String>,
+}
+
 #[api(
     input: {
         properties: {},
@@ -97,12 +107,13 @@ impl Drop for UnmountGuard {
 pub(crate) fn do_create_datastore(
     _lock: BackupLockGuard,
     mut config: SectionConfigData,
-    datastore: DataStoreConfig,
+    mut datastore: DataStoreConfig,
     reuse_datastore: bool,
+    overwrite_in_use: bool,
 ) -> Result<(), Error> {
     let path: PathBuf = datastore.absolute_path().into();
 
-    if path.parent().is_none() {
+    if path.parent().is_none() && datastore.backing_device.is_none() {
         bail!("cannot create datastore in root path");
     }
 
@@ -116,6 +127,35 @@ pub(crate) fn do_create_datastore(
             .parse_property_string(datastore.tuning.as_deref().unwrap_or(""))?,
     )?;
 
+    let (backend_type, backend_s3_client) =
+        match DataStore::s3_client_and_backend_from_datastore_config(&datastore)? {
+            (backend_type, Some(s3_client)) => {
+                if !overwrite_in_use {
+                    let object_key = S3ObjectKey::try_from(S3_DATASTORE_IN_USE_MARKER)
+                        .context("failed to generate s3 object key")?;
+                    if let Some(response) =
+                        proxmox_async::runtime::block_on(s3_client.get_object(object_key.clone()))
+                            .context("failed to get in-use marker from bucket")?
+                    {
+                        let content = proxmox_async::runtime::block_on(response.content.collect())
+                            .unwrap_or_default();
+                        let content =
+                            String::from_utf8(content.to_bytes().to_vec()).unwrap_or_default();
+                        let in_use: InUseContent =
+                            serde_json::from_str(&content).unwrap_or_default();
+                        if let Some(hostname) = in_use.hostname {
+                            bail!("Bucket already contains datastore in use by host {hostname}");
+                        } else {
+                            bail!("Bucket already contains datastore in use");
+                        }
+                    }
+                }
+
+                (backend_type, Some(Arc::new(s3_client)))
+            }
+            (backend_type, None) => (backend_type, None),
+        };
+
     let unmount_guard = if datastore.backing_device.is_some() {
         do_mount_device(datastore.clone())?;
         UnmountGuard::new(Some(path.clone()))
@@ -123,7 +163,7 @@ pub(crate) fn do_create_datastore(
         UnmountGuard::new(None)
     };
 
-    let chunk_store = if reuse_datastore {
+    let chunk_store = if reuse_datastore && backend_type == DatastoreBackendType::Filesystem {
         ChunkStore::verify_chunkstore(&path).and_then(|_| {
             // Must be the only instance accessing and locking the chunk store,
             // dropping will close all other locks from this process on the lockfile as well.
@@ -134,15 +174,33 @@ pub(crate) fn do_create_datastore(
             )
         })?
     } else {
-        if let Ok(dir) = std::fs::read_dir(&path) {
-            for file in dir {
-                let name = file?.file_name();
-                let name = name.to_str();
-                if !name.is_some_and(|name| name.starts_with('.') || name == "lost+found") {
-                    bail!("datastore path not empty");
+        if !reuse_datastore && backend_type == DatastoreBackendType::Filesystem {
+            if let Ok(dir) = std::fs::read_dir(&path) {
+                for file in dir {
+                    let name = file?.file_name();
+                    let name = name.to_str();
+                    if !name.is_some_and(|name| name.starts_with('.') || name == "lost+found") {
+                        bail!("datastore path not empty");
+                    }
                 }
             }
         }
+        if reuse_datastore && backend_type == DatastoreBackendType::S3 {
+            let chunks_path = path.join(".chunks");
+            if let Err(err) = std::fs::remove_dir_all(&chunks_path) {
+                if err.kind() != std::io::ErrorKind::NotFound {
+                    return Err(err).with_context(|| {
+                        format!("failed to remove pre-existing chunks in {chunks_path:?}")
+                    });
+                }
+            }
+            // starting out in maintenance mode s3-refresh,
+            // so no other operation will start until done with that.
+            datastore.set_maintenance_mode(Some(MaintenanceMode {
+                ty: MaintenanceType::S3Refresh,
+                message: None,
+            }))?;
+        }
         let backup_user = pbs_config::backup_user()?;
         ChunkStore::create(
             &datastore.name,
@@ -153,9 +211,25 @@ pub(crate) fn do_create_datastore(
         )?
     };
 
+    if let Some(ref s3_client) = backend_s3_client {
+        let object_key = S3ObjectKey::try_from(S3_DATASTORE_IN_USE_MARKER)
+            .context("failed to generate s3 object key")?;
+        let content = serde_json::to_string(&InUseContent {
+            hostname: Some(proxmox_sys::nodename().to_string()),
+        })
+        .context("failed to encode hostname")?;
+        proxmox_async::runtime::block_on(s3_client.put_object(
+            object_key,
+            hyper::body::Bytes::from(content).into(),
+            Some(S3_HTTP_REQUEST_TIMEOUT),
+            true,
+        ))
+        .context("failed to upload in-use marker for datastore")?;
+    }
+
     if tuning.gc_atime_safety_check.unwrap_or(true) {
         chunk_store
-            .check_fs_atime_updates(true)
+            .check_fs_atime_updates(true, backend_s3_client)
             .context("access time safety check failed")?;
         info!("Access time update check successful.");
     } else {
@@ -186,7 +260,13 @@ pub(crate) fn do_create_datastore(
                 optional: true,
                 default: false,
                 description: "Re-use existing datastore directory."
-            }
+            },
+            "overwrite-in-use": {
+                type: Boolean,
+                optional: true,
+                default: false,
+                description: "Overwrite in use marker (S3 backed datastores only)."
+            },
         },
     },
     access: {
@@ -198,6 +278,7 @@ pub(crate) fn do_create_datastore(
 pub fn create_datastore(
     config: DataStoreConfig,
     reuse_datastore: bool,
+    overwrite_in_use: bool,
     rpcenv: &mut dyn RpcEnvironment,
 ) -> Result<String, Error> {
     let lock = pbs_config::datastore::lock_config()?;
@@ -259,19 +340,37 @@ pub fn create_datastore(
         ..config
     };
 
+    let store_name = config.name.to_string();
+
+    let (backend, s3_client) = DataStore::s3_client_and_backend_from_datastore_config(&config)?;
+    if let Some(s3_client) = s3_client {
+        proxmox_async::runtime::block_on(s3_client.head_bucket())
+            .context("failed to access bucket")
+            .map_err(|err| format_err!("{err:#}"))?;
+    }
+
     WorkerTask::new_thread(
         "create-datastore",
-        Some(config.name.to_string()),
+        Some(store_name.clone()),
         auth_id.to_string(),
         to_stdout,
-        move |_worker| {
-            do_create_datastore(lock, section_config, config, reuse_datastore)?;
+        move |worker| {
+            do_create_datastore(
+                lock,
+                section_config,
+                config,
+                reuse_datastore,
+                overwrite_in_use,
+            )?;
 
             if let Some(prune_job_config) = prune_job_config {
-                do_create_prune_job(prune_job_config)
-            } else {
-                Ok(())
+                do_create_prune_job(prune_job_config)?;
             }
+
+            if reuse_datastore && backend == DatastoreBackendType::S3 {
+                crate::api2::admin::datastore::do_s3_refresh(&store_name, &worker)?;
+            }
+            Ok(())
         },
     )
 }
@@ -625,7 +724,12 @@ pub async fn delete_datastore(
         for job in list_verification_jobs(Some(name.clone()), Value::Null, rpcenv)? {
             delete_verification_job(job.config.id, None, rpcenv)?
         }
-        for job in list_config_sync_jobs(Some(name.clone()), None, Value::Null, rpcenv)? {
+        for job in list_config_sync_jobs(
+            Some(name.clone()),
+            Some(ListSyncDirection::All),
+            Value::Null,
+            rpcenv,
+        )? {
             delete_sync_job(job.config.id, None, rpcenv)?
         }
         for job in list_prune_jobs(Some(name.clone()), Value::Null, rpcenv)? {
@@ -633,7 +737,7 @@ pub async fn delete_datastore(
         }
 
         let (mut tree, _digest) = pbs_config::acl::config()?;
-        tree.delete_node(&format!("/datastore/{}", name));
+        tree.delete_node(&format!("/datastore/{name}"));
         pbs_config::acl::save_config(&tree)?;
 
         let tape_jobs = list_tape_backup_jobs(Value::Null, rpcenv)?;
