@@ -339,6 +339,10 @@ pub enum DeletableProperty {
     EncryptedOnly,
     /// Delete the verified_only property,
     VerifiedOnly,
+    /// Delete the run_on_mount property,
+    RunOnMount,
+    /// Delete the unmount_on_done property,
+    UnmountOnDone,
     /// Delete the sync_direction property,
     SyncDirection,
 }
@@ -458,6 +462,12 @@ pub fn update_sync_job(
                 DeletableProperty::VerifiedOnly => {
                     data.verified_only = None;
                 }
+                DeletableProperty::RunOnMount => {
+                    data.run_on_mount = None;
+                }
+                DeletableProperty::UnmountOnDone => {
+                    data.unmount_on_done = None;
+                }
                 DeletableProperty::SyncDirection => {
                     data.sync_direction = None;
                 }
@@ -507,6 +517,15 @@ pub fn update_sync_job(
     if let Some(verified_only) = update.verified_only {
         data.verified_only = Some(verified_only);
     }
+    if let Some(run_on_mount) = update.run_on_mount {
+        data.run_on_mount = Some(run_on_mount);
+    }
+    if let Some(unmount_on_done) = update.unmount_on_done {
+        if unmount_on_done && data.run_on_mount != Some(true) {
+            bail!("'unmount-on-done' can only be set when 'run-on-mount' is enabled");
+        }
+        data.unmount_on_done = Some(unmount_on_done);
+    }
     if let Some(sync_direction) = update.sync_direction {
         data.sync_direction = Some(sync_direction);
     }
@@ -683,6 +702,8 @@ acl:1:/remote/remote1/remotestore1:write@pbs:RemoteSyncOperator
         transfer_last: None,
         encrypted_only: None,
         verified_only: None,
+        run_on_mount: None,
+        unmount_on_done: None,
         sync_direction: None, // use default
     };
 
