@@ -5,27 +5,26 @@ use std::str::FromStr;
 use anyhow::{format_err, Error};
 use serde_json::{json, Value};
 
-use proxmox_router::{cli::*, RpcEnvironment};
-use proxmox_schema::api;
-use proxmox_sys::fs::CreateOptions;
-
 use pbs_api_types::percent_encoding::percent_encode_component;
 use pbs_api_types::{
     BackupNamespace, GroupFilter, RateLimitConfig, SyncDirection, SyncJobConfig, DATASTORE_SCHEMA,
     GROUP_FILTER_LIST_SCHEMA, IGNORE_VERIFIED_BACKUPS_SCHEMA, NS_MAX_DEPTH_SCHEMA,
     REMOTE_ID_SCHEMA, REMOVE_VANISHED_BACKUPS_SCHEMA, RESYNC_CORRUPT_SCHEMA,
     SYNC_ENCRYPTED_ONLY_SCHEMA, SYNC_VERIFIED_ONLY_SCHEMA, TRANSFER_LAST_SCHEMA, UPID_SCHEMA,
-    VERIFICATION_OUTDATED_AFTER_SCHEMA,
+    VERIFICATION_OUTDATED_AFTER_SCHEMA, VERIFY_JOB_READ_THREADS_SCHEMA,
+    VERIFY_JOB_VERIFY_THREADS_SCHEMA,
 };
+use proxmox_rest_server::wait_for_local_worker;
+use proxmox_router::{cli::*, RpcEnvironment};
+use proxmox_schema::api;
+use proxmox_sys::fs::CreateOptions;
+
+use pbs_buildcfg::configdir;
 use pbs_client::{display_task_log, view_task_result};
 use pbs_config::sync;
 use pbs_tools::json::required_string_param;
-
-use proxmox_rest_server::wait_for_local_worker;
-
 use proxmox_backup::api2;
 use proxmox_backup::client_helpers::connect_to_localhost;
-use proxmox_backup::config;
 
 mod proxmox_backup_manager;
 use proxmox_backup_manager::*;
@@ -51,7 +50,7 @@ async fn start_garbage_collection(param: Value) -> Result<Value, Error> {
 
     let client = connect_to_localhost()?;
 
-    let path = format!("api2/json/admin/datastore/{}/gc", store);
+    let path = format!("api2/json/admin/datastore/{store}/gc");
 
     let result = client.post(&path, None).await?;
 
@@ -81,7 +80,7 @@ async fn garbage_collection_status(param: Value) -> Result<Value, Error> {
 
     let client = connect_to_localhost()?;
 
-    let path = format!("api2/json/admin/datastore/{}/gc", store);
+    let path = format!("api2/json/admin/datastore/{store}/gc");
 
     let mut result = client.get(&path, None).await?;
     let mut data = result["data"].take();
@@ -583,6 +582,14 @@ async fn push_datastore(
                 schema: OUTPUT_FORMAT,
                 optional: true,
             },
+            "read-threads": {
+                schema: VERIFY_JOB_READ_THREADS_SCHEMA,
+                optional: true,
+            },
+            "verify-threads": {
+                schema: VERIFY_JOB_VERIFY_THREADS_SCHEMA,
+                optional: true,
+            },
         }
    }
 )]
@@ -594,7 +601,7 @@ async fn verify(store: String, mut param: Value) -> Result<Value, Error> {
 
     let args = json!(param);
 
-    let path = format!("api2/json/admin/datastore/{}/verify", store);
+    let path = format!("api2/json/admin/datastore/{store}/verify");
 
     let result = client.post(&path, Some(args)).await?;
 
@@ -659,6 +666,8 @@ async fn run() -> Result<(), Error> {
         .tasklog_pbs()
         .init()?;
     proxmox_backup::server::notifications::init()?;
+    proxmox_product_config::init(pbs_config::backup_user()?, pbs_config::priv_user()?);
+    proxmox_acme_api::init(configdir!("/acme"), false)?;
 
     let cmd_def = CliCommandMap::new()
         .insert("acl", acl_commands())
@@ -677,6 +686,7 @@ async fn run() -> Result<(), Error> {
         .insert("garbage-collection", garbage_collection_commands())
         .insert("acme", acme_mgmt_cli())
         .insert("cert", cert_mgmt_cli())
+        .insert("s3", s3_commands())
         .insert("subscription", subscription_commands())
         .insert("sync-job", sync_job_commands())
         .insert("verify-job", verify_job_commands())
@@ -712,9 +722,9 @@ async fn run() -> Result<(), Error> {
         .insert("report", CliCommand::new(&API_METHOD_REPORT))
         .insert("versions", CliCommand::new(&API_METHOD_GET_VERSIONS));
 
-    let args: Vec<String> = std::env::args().take(2).collect();
-    if args.len() >= 2 && args[1] == "update-to-prune-jobs-config" {
-        return update_to_prune_jobs_config();
+    let args: Vec<String> = std::env::args().take(3).collect();
+    if args.len() >= 3 && args[1] == "migrate-config" {
+        return migrate_config::handle_command(&args[2]);
     }
     let avoid_init = args.len() >= 2 && (args[1] == "bashcomplete" || args[1] == "printdoc");
 
@@ -756,7 +766,7 @@ async fn run_job(job_type: &str, param: Value) -> Result<Value, Error> {
 
     let client = connect_to_localhost()?;
 
-    let path = format!("api2/json/admin/{}/{}/run", job_type, id);
+    let path = format!("api2/json/admin/{job_type}/{id}/run");
     let result = client.post(&path, None).await?;
     view_task_result(&client, result, &output_format).await?;
 
@@ -949,7 +959,7 @@ pub fn complete_remote_datastore_group_filter(
     list.extend(
         complete_remote_datastore_group(_arg, param)
             .iter()
-            .map(|group| format!("group:{}", group)),
+            .map(|group| format!("group:{group}")),
     );
 
     list
