@@ -7,26 +7,27 @@ use futures::*;
 use hyper::header;
 use hyper::http::request::Parts;
 use hyper::http::Response;
-use hyper::{Body, StatusCode};
+use hyper::StatusCode;
+use hyper_util::server::graceful::GracefulShutdown;
+use openssl::ssl::SslAcceptor;
+use serde_json::{json, Value};
 use tracing::level_filters::LevelFilter;
 use tracing::{info, warn};
 use url::form_urlencoded;
 
-use openssl::ssl::SslAcceptor;
-use serde_json::{json, Value};
-
+use proxmox_http::Body;
+use proxmox_http::RateLimiterTag;
 use proxmox_lang::try_block;
+use proxmox_rest_server::{
+    cleanup_old_tasks, cookie_from_header, rotate_task_log_archive, ApiConfig, Redirector,
+    RestEnvironment, RestServer, WorkerTask,
+};
 use proxmox_router::{RpcEnvironment, RpcEnvironmentType};
 use proxmox_sys::fs::CreateOptions;
 use proxmox_sys::logrotate::LogRotate;
 
 use pbs_datastore::DataStore;
 
-use proxmox_rest_server::{
-    cleanup_old_tasks, cookie_from_header, rotate_task_log_archive, ApiConfig, Redirector,
-    RestEnvironment, RestServer, WorkerTask,
-};
-
 use proxmox_backup::{
     server::{
         auth::check_pbs_auth,
@@ -186,6 +187,8 @@ async fn run() -> Result<(), Error> {
     proxmox_backup::auth_helpers::setup_auth_context(false);
     proxmox_backup::server::notifications::init()?;
     metric_collection::init()?;
+    proxmox_product_config::init(pbs_config::backup_user()?, pbs_config::priv_user()?);
+    proxmox_acme_api::init(configdir!("/acme"), false)?;
 
     let mut indexpath = PathBuf::from(pbs_buildcfg::JS_DIR);
     indexpath.push("index.hbs");
@@ -193,6 +196,7 @@ async fn run() -> Result<(), Error> {
     let mut config = ApiConfig::new(pbs_buildcfg::JS_DIR, RpcEnvironmentType::PUBLIC)
         .index_handler_func(|e, p| Box::pin(get_index_future(e, p)))
         .auth_handler_func(|h, m| Box::pin(check_pbs_auth(h, m)))
+        .auth_cookie_name(proxmox_backup::auth_helpers::get_auth_cookie_name())
         .register_template("index", &indexpath)?
         .register_template("console", "/usr/share/pve-xtermjs/index.html.hbs")?
         .default_api2_handler(&proxmox_backup::api2::ROUTER)
@@ -289,27 +293,71 @@ async fn run() -> Result<(), Error> {
     let server = proxmox_daemon::server::create_daemon(
         ([0, 0, 0, 0, 0, 0, 0, 0], 8007).into(),
         move |listener| {
-            let (secure_connections, insecure_connections) =
+            let (mut secure_connections, mut insecure_connections) =
                 connections.accept_tls_optional(listener, acceptor);
 
             Ok(async {
                 proxmox_systemd::notify::SystemdNotify::Ready.notify()?;
 
-                let secure_server = hyper::Server::builder(secure_connections)
-                    .serve(rest_server)
-                    .with_graceful_shutdown(proxmox_daemon::shutdown_future())
-                    .map_err(Error::from);
+                let secure_server = async move {
+                    let graceful = GracefulShutdown::new();
+                    loop {
+                        tokio::select! {
+                            Some(conn) = secure_connections.next() => {
+                                match conn {
+                                    Ok(conn) => {
+                                        match rest_server.api_service(&conn) {
+                                            Ok(api_service) => {
+                                                let watcher = graceful.watcher();
+                                                tokio::spawn(async move {
+                                                    api_service.serve(conn, Some(watcher)).await
+                                                });
+                                            }
+                                            Err(err) => log::warn!("Failed to get api service: {err:?}"),
+                                        }
+                                    },
+                                    Err(err) => { log::warn!("Failed to accept secure connection: {err:?}"); }
+                                }
+                            },
+                            _shutdown = proxmox_daemon::shutdown_future() => {
+                                break;
+                            }
+                        }
+                    }
+                    graceful.shutdown().await;
+                    Ok::<(), Error>(())
+                };
 
-                let insecure_server = hyper::Server::builder(insecure_connections)
-                    .serve(redirector)
-                    .with_graceful_shutdown(proxmox_daemon::shutdown_future())
-                    .map_err(Error::from);
+                let insecure_server = async move {
+                    let graceful = GracefulShutdown::new();
+                    loop {
+                        tokio::select! {
+                            Some(conn) = insecure_connections.next() => {
+                                match conn {
+                                    Ok(conn) => {
+                                        let redirect_service = redirector.redirect_service();
+                                        let watcher = graceful.watcher();
+                                        tokio::spawn(async move {
+                                            redirect_service.serve(conn, Some(watcher)).await
+                                        });
+                                    },
+                                    Err(err) => { log::warn!("Failed to accept insecure connection: {err:?}"); }
+                                }
+                            },
+                            _shutdown = proxmox_daemon::shutdown_future() => {
+                                break;
+                            }
+                        }
+                    }
+                    graceful.shutdown().await;
+                    Ok::<(), Error>(())
+                };
 
                 let (secure_res, insecure_res) =
                     try_join!(tokio::spawn(secure_server), tokio::spawn(insecure_server))
                         .context("failed to complete REST server task")?;
 
-                let results = [secure_res, insecure_res];
+                let results: [Result<(), Error>; 2] = [secure_res, insecure_res];
 
                 if results.iter().any(Result::is_err) {
                     let cat_errors = results
@@ -321,7 +369,7 @@ async fn run() -> Result<(), Error> {
                     bail!(cat_errors);
                 }
 
-                Ok(())
+                Ok::<(), Error>(())
             })
         },
         Some(pbs_buildcfg::PROXMOX_BACKUP_PROXY_PID_FN),
@@ -907,6 +955,7 @@ async fn run_traffic_control_updater() {
 
 fn lookup_rate_limiter(
     peer: std::net::SocketAddr,
+    tags: &[RateLimiterTag],
 ) -> (Option<SharedRateLimit>, Option<SharedRateLimit>) {
     let mut cache = TRAFFIC_CONTROL_CACHE.lock().unwrap();
 
@@ -914,7 +963,15 @@ fn lookup_rate_limiter(
 
     cache.reload(now);
 
-    let (_rule_name, read_limiter, write_limiter) = cache.lookup_rate_limiter(peer, now);
+    let user = tags.iter().find_map(|tag| match tag {
+        RateLimiterTag::User(user) => Some(user.as_str()),
+    });
+
+    let authid = user.and_then(|s| s.parse::<pbs_api_types::Authid>().ok());
+    let user_parsed = authid.as_ref().map(|auth_id| auth_id.user());
+
+    let (_rule_name, read_limiter, write_limiter) =
+        cache.lookup_rate_limiter(peer, now, user_parsed);
 
     (read_limiter, write_limiter)
 }
