@@ -8,31 +8,34 @@ mod cert;
 pub use cert::*;
 mod datastore;
 pub use datastore::*;
+mod disk;
+pub use disk::*;
 mod dns;
 pub use dns::*;
 mod ldap;
 pub use ldap::*;
+pub mod migrate_config;
 mod network;
 pub use network::*;
-mod prune;
-pub use prune::*;
-mod remote;
-pub use remote::*;
-mod sync;
-pub use sync::*;
-mod verify;
-pub use verify::*;
-mod user;
-pub use user::*;
-mod subscription;
-pub use subscription::*;
-mod disk;
-pub use disk::*;
 mod node;
 pub use node::*;
 mod notifications;
 pub use notifications::*;
 mod openid;
 pub use openid::*;
+mod prune;
+pub use prune::*;
+mod remote;
+pub use remote::*;
+mod s3;
+pub use s3::*;
+mod subscription;
+pub use subscription::*;
+mod sync;
+pub use sync::*;
 mod traffic_control;
 pub use traffic_control::*;
+mod user;
+pub use user::*;
+mod verify;
+pub use verify::*;
