@@ -1,20 +1,21 @@
 //! Sync datastore by pulling contents from remote server
 
-use std::collections::HashSet;
+use std::collections::hash_map::Entry;
+use std::collections::{HashMap, HashSet};
 use std::io::Seek;
 use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::{Arc, Mutex};
 use std::time::SystemTime;
 
-use anyhow::{bail, format_err, Error};
+use anyhow::{bail, format_err, Context, Error};
 use proxmox_human_byte::HumanByte;
-use tracing::info;
+use tracing::{info, warn};
 
 use pbs_api_types::{
     print_store_and_ns, ArchiveType, Authid, BackupArchiveName, BackupDir, BackupGroup,
-    BackupNamespace, GroupFilter, Operation, RateLimitConfig, Remote, VerifyState,
-    CLIENT_LOG_BLOB_NAME, MANIFEST_BLOB_NAME, MAX_NAMESPACE_DEPTH, PRIV_DATASTORE_AUDIT,
-    PRIV_DATASTORE_BACKUP,
+    BackupNamespace, GroupFilter, Operation, RateLimitConfig, Remote, SnapshotListItem,
+    VerifyState, CLIENT_LOG_BLOB_NAME, MANIFEST_BLOB_NAME, MAX_NAMESPACE_DEPTH,
+    PRIV_DATASTORE_AUDIT, PRIV_DATASTORE_BACKUP,
 };
 use pbs_client::BackupRepository;
 use pbs_config::CachedUserInfo;
@@ -24,12 +25,13 @@ use pbs_datastore::fixed_index::FixedIndexReader;
 use pbs_datastore::index::IndexFile;
 use pbs_datastore::manifest::{BackupManifest, FileInfo};
 use pbs_datastore::read_chunk::AsyncReadChunk;
-use pbs_datastore::{check_backup_owner, DataStore, StoreProgress};
+use pbs_datastore::{check_backup_owner, DataStore, DatastoreBackend, StoreProgress};
 use pbs_tools::sha::sha256;
 
 use super::sync::{
-    check_namespace_depth_limit, ignore_not_verified_or_encrypted, LocalSource, RemoteSource,
-    RemovedVanishedStats, SkipInfo, SkipReason, SyncSource, SyncSourceReader, SyncStats,
+    check_namespace_depth_limit, exclude_not_verified_or_encrypted,
+    ignore_not_verified_or_encrypted, LocalSource, RemoteSource, RemovedVanishedStats, SkipInfo,
+    SkipReason, SyncSource, SyncSourceReader, SyncStats,
 };
 use crate::backup::{check_ns_modification_privs, check_ns_privs};
 use crate::tools::parallel_handler::ParallelHandler;
@@ -37,6 +39,8 @@ use crate::tools::parallel_handler::ParallelHandler;
 pub(crate) struct PullTarget {
     store: Arc<DataStore>,
     ns: BackupNamespace,
+    // Contains the active S3Client in case of S3 backend
+    backend: DatastoreBackend,
 }
 
 /// Parameters for a pull operation.
@@ -113,10 +117,9 @@ impl PullParameters {
                 ns: remote_ns,
             })
         };
-        let target = PullTarget {
-            store: DataStore::lookup_datastore(store, Some(Operation::Write))?,
-            ns,
-        };
+        let store = DataStore::lookup_datastore(store, Some(Operation::Write))?;
+        let backend = store.backend()?;
+        let target = PullTarget { store, ns, backend };
 
         let group_filter = group_filter.unwrap_or_default();
 
@@ -139,7 +142,8 @@ async fn pull_index_chunks<I: IndexFile>(
     chunk_reader: Arc<dyn AsyncReadChunk>,
     target: Arc<DataStore>,
     index: I,
-    downloaded_chunks: Arc<Mutex<HashSet<[u8; 32]>>>,
+    encountered_chunks: Arc<Mutex<EncounteredChunks>>,
+    backend: &DatastoreBackend,
 ) -> Result<SyncStats, Error> {
     use futures::stream::{self, StreamExt, TryStreamExt};
 
@@ -149,25 +153,23 @@ async fn pull_index_chunks<I: IndexFile>(
         (0..index.index_count())
             .map(|pos| index.chunk_info(pos).unwrap())
             .filter(|info| {
-                let mut guard = downloaded_chunks.lock().unwrap();
-                let done = guard.contains(&info.digest);
-                if !done {
-                    // Note: We mark a chunk as downloaded before its actually downloaded
-                    // to avoid duplicate downloads.
-                    guard.insert(info.digest);
+                let guard = encountered_chunks.lock().unwrap();
+                match guard.check_reusable(&info.digest) {
+                    Some(touched) => !touched, // reusable and already touched, can always skip
+                    None => true,
                 }
-                !done
             }),
     );
 
     let target2 = target.clone();
+    let backend = backend.clone();
     let verify_pool = ParallelHandler::new(
         "sync chunk writer",
         4,
         move |(chunk, digest, size): (DataBlob, [u8; 32], u64)| {
             // println!("verify and write {}", hex::encode(&digest));
             chunk.verify_unencrypted(size as usize, &digest)?;
-            target2.insert_chunk(&chunk, &digest)?;
+            target2.insert_chunk(&chunk, &digest, &backend)?;
             Ok(())
         },
     );
@@ -184,15 +186,30 @@ async fn pull_index_chunks<I: IndexFile>(
             let bytes = Arc::clone(&bytes);
             let chunk_count = Arc::clone(&chunk_count);
             let verify_and_write_channel = verify_and_write_channel.clone();
+            let encountered_chunks = Arc::clone(&encountered_chunks);
 
             Ok::<_, Error>(async move {
-                let chunk_exists = proxmox_async::runtime::block_in_place(|| {
-                    target.cond_touch_chunk(&info.digest, false)
-                })?;
-                if chunk_exists {
-                    //info!("chunk {} exists {}", pos, hex::encode(digest));
-                    return Ok::<_, Error>(());
+                {
+                    // limit guard scope
+                    let mut guard = encountered_chunks.lock().unwrap();
+                    if let Some(touched) = guard.check_reusable(&info.digest) {
+                        if touched {
+                            return Ok::<_, Error>(());
+                        }
+                        let chunk_exists = proxmox_async::runtime::block_in_place(|| {
+                            target.cond_touch_chunk(&info.digest, false)
+                        })?;
+                        if chunk_exists {
+                            guard.mark_touched(&info.digest);
+                            //info!("chunk {} exists {}", pos, hex::encode(digest));
+                            return Ok::<_, Error>(());
+                        }
+                    }
+                    // mark before actually downloading the chunk, so this happens only once
+                    guard.mark_reusable(&info.digest);
+                    guard.mark_touched(&info.digest);
                 }
+
                 //info!("sync {} chunk {}", pos, hex::encode(digest));
                 let chunk = chunk_reader.read_raw_chunk(&info.digest).await?;
                 let raw_size = chunk.raw_size() as usize;
@@ -214,7 +231,7 @@ async fn pull_index_chunks<I: IndexFile>(
 
     drop(verify_and_write_channel);
 
-    verify_pool.complete()?;
+    tokio::task::spawn_blocking(|| verify_pool.complete()).await??;
 
     let elapsed = start_time.elapsed()?;
 
@@ -264,7 +281,8 @@ async fn pull_single_archive<'a>(
     reader: Arc<dyn SyncSourceReader + 'a>,
     snapshot: &'a pbs_datastore::BackupDir,
     archive_info: &'a FileInfo,
-    downloaded_chunks: Arc<Mutex<HashSet<[u8; 32]>>>,
+    encountered_chunks: Arc<Mutex<EncounteredChunks>>,
+    backend: &DatastoreBackend,
 ) -> Result<SyncStats, Error> {
     let archive_name = &archive_info.filename;
     let mut path = snapshot.full_path();
@@ -293,10 +311,13 @@ async fn pull_single_archive<'a>(
                 info!("skipping chunk sync for same datastore");
             } else {
                 let stats = pull_index_chunks(
-                    reader.chunk_reader(archive_info.crypt_mode),
+                    reader
+                        .chunk_reader(archive_info.crypt_mode)
+                        .context("failed to get chunk reader")?,
                     snapshot.datastore().clone(),
                     index,
-                    downloaded_chunks,
+                    encountered_chunks,
+                    backend,
                 )
                 .await?;
                 sync_stats.add(stats);
@@ -313,10 +334,13 @@ async fn pull_single_archive<'a>(
                 info!("skipping chunk sync for same datastore");
             } else {
                 let stats = pull_index_chunks(
-                    reader.chunk_reader(archive_info.crypt_mode),
+                    reader
+                        .chunk_reader(archive_info.crypt_mode)
+                        .context("failed to get chunk reader")?,
                     snapshot.datastore().clone(),
                     index,
-                    downloaded_chunks,
+                    encountered_chunks,
+                    backend,
                 )
                 .await?;
                 sync_stats.add(stats);
@@ -331,6 +355,11 @@ async fn pull_single_archive<'a>(
     if let Err(err) = std::fs::rename(&tmp_path, &path) {
         bail!("Atomic rename file {:?} failed - {}", path, err);
     }
+
+    backend
+        .upload_index_to_backend(snapshot, archive_name)
+        .await?;
+
     Ok(sync_stats)
 }
 
@@ -347,7 +376,7 @@ async fn pull_snapshot<'a>(
     params: &PullParameters,
     reader: Arc<dyn SyncSourceReader + 'a>,
     snapshot: &'a pbs_datastore::BackupDir,
-    downloaded_chunks: Arc<Mutex<HashSet<[u8; 32]>>>,
+    encountered_chunks: Arc<Mutex<EncounteredChunks>>,
     corrupt: bool,
     is_new: bool,
 ) -> Result<SyncStats, Error> {
@@ -401,6 +430,7 @@ async fn pull_snapshot<'a>(
         }
     }
 
+    let manifest_data = tmp_manifest_blob.raw_data().to_vec();
     let manifest = BackupManifest::try_from(tmp_manifest_blob)?;
 
     if ignore_not_verified_or_encrypted(
@@ -419,6 +449,7 @@ async fn pull_snapshot<'a>(
         return Ok(sync_stats);
     }
 
+    let backend = &params.target.backend;
     for item in manifest.files() {
         let mut path = snapshot.full_path();
         path.push(&item.filename);
@@ -459,17 +490,54 @@ async fn pull_snapshot<'a>(
             }
         }
 
-        let stats =
-            pull_single_archive(reader.clone(), snapshot, item, downloaded_chunks.clone()).await?;
+        let stats = pull_single_archive(
+            reader.clone(),
+            snapshot,
+            item,
+            encountered_chunks.clone(),
+            backend,
+        )
+        .await?;
         sync_stats.add(stats);
     }
 
     if let Err(err) = std::fs::rename(&tmp_manifest_name, &manifest_name) {
         bail!("Atomic rename file {:?} failed - {}", manifest_name, err);
     }
+    if let DatastoreBackend::S3(s3_client) = backend {
+        let object_key = pbs_datastore::s3::object_key_from_path(
+            &snapshot.relative_path(),
+            MANIFEST_BLOB_NAME.as_ref(),
+        )
+        .context("invalid manifest object key")?;
+
+        let data = hyper::body::Bytes::from(manifest_data);
+        let _is_duplicate = s3_client
+            .upload_replace_with_retry(object_key, data)
+            .await
+            .context("failed to upload manifest to s3 backend")?;
+    }
 
     if !client_log_name.exists() {
         reader.try_download_client_log(&client_log_name).await?;
+        if client_log_name.exists() {
+            if let DatastoreBackend::S3(s3_client) = backend {
+                let object_key = pbs_datastore::s3::object_key_from_path(
+                    &snapshot.relative_path(),
+                    CLIENT_LOG_BLOB_NAME.as_ref(),
+                )
+                .context("invalid archive object key")?;
+
+                let data = tokio::fs::read(&client_log_name)
+                    .await
+                    .context("failed to read log file contents")?;
+                let contents = hyper::body::Bytes::from(data);
+                let _is_duplicate = s3_client
+                    .upload_replace_with_retry(object_key, contents)
+                    .await
+                    .context("failed to upload client log to s3 backend")?;
+            }
+        }
     };
     snapshot
         .cleanup_unreferenced_files(&manifest)
@@ -486,14 +554,22 @@ async fn pull_snapshot_from<'a>(
     params: &PullParameters,
     reader: Arc<dyn SyncSourceReader + 'a>,
     snapshot: &'a pbs_datastore::BackupDir,
-    downloaded_chunks: Arc<Mutex<HashSet<[u8; 32]>>>,
+    encountered_chunks: Arc<Mutex<EncounteredChunks>>,
     corrupt: bool,
 ) -> Result<SyncStats, Error> {
     let (_path, is_new, _snap_lock) = snapshot
         .datastore()
         .create_locked_backup_dir(snapshot.backup_ns(), snapshot.as_ref())?;
 
-    let result = pull_snapshot(params, reader, snapshot, downloaded_chunks, corrupt, is_new).await;
+    let result = pull_snapshot(
+        params,
+        reader,
+        snapshot,
+        encountered_chunks,
+        corrupt,
+        is_new,
+    )
+    .await;
 
     if is_new {
         // Cleanup directory on error if snapshot was not present before
@@ -537,22 +613,16 @@ async fn pull_group(
     source_namespace: &BackupNamespace,
     group: &BackupGroup,
     progress: &mut StoreProgress,
+    encountered_chunks: Arc<Mutex<EncounteredChunks>>,
 ) -> Result<SyncStats, Error> {
     let mut already_synced_skip_info = SkipInfo::new(SkipReason::AlreadySynced);
     let mut transfer_last_skip_info = SkipInfo::new(SkipReason::TransferLast);
 
-    let mut raw_list: Vec<BackupDir> = params
+    let mut raw_list: Vec<SnapshotListItem> = params
         .source
-        .list_backup_dirs(source_namespace, group)
+        .list_backup_snapshots(source_namespace, group)
         .await?;
-    raw_list.sort_unstable_by(|a, b| a.time.cmp(&b.time));
-
-    let total_amount = raw_list.len();
-
-    let cutoff = params
-        .transfer_last
-        .map(|count| total_amount.saturating_sub(count))
-        .unwrap_or_default();
+    raw_list.sort_unstable_by(|a, b| a.backup.time.cmp(&b.backup.time));
 
     let target_ns = source_namespace.map_prefix(&params.source.get_ns(), &params.target.ns)?;
 
@@ -567,8 +637,13 @@ async fn pull_group(
     // Also stores if the snapshot is corrupt (verification job failed)
     let list: Vec<(BackupDir, bool)> = raw_list
         .into_iter()
-        .enumerate()
-        .filter_map(|(pos, dir)| {
+        .filter_map(|item| {
+            if exclude_not_verified_or_encrypted(&item, params.verified_only, params.encrypted_only)
+            {
+                return None;
+            }
+
+            let dir = item.backup;
             source_snapshots.insert(dir.time);
             // If resync_corrupt is set, check if the corresponding local snapshot failed to
             // verification
@@ -598,6 +673,23 @@ async fn pull_group(
                     }
                 }
             }
+            Some((dir, false))
+        })
+        .collect();
+
+    let total_amount = list.len();
+    let cutoff = params
+        .transfer_last
+        .map(|count| total_amount.saturating_sub(count))
+        .unwrap_or_default();
+
+    let list: Vec<(BackupDir, bool)> = list
+        .into_iter()
+        .enumerate()
+        .filter_map(|(pos, (dir, needs_resync))| {
+            if params.resync_corrupt && needs_resync {
+                return Some((dir, needs_resync));
+            }
             // Note: the snapshot represented by `last_sync_time` might be missing its backup log
             // or post-backup verification state if those were not yet available during the last
             // sync run, always resync it
@@ -609,7 +701,7 @@ async fn pull_group(
                 transfer_last_skip_info.update(dir.time);
                 return None;
             }
-            Some((dir, false))
+            Some((dir, needs_resync))
         })
         .collect();
 
@@ -622,8 +714,64 @@ async fn pull_group(
         transfer_last_skip_info.reset();
     }
 
-    // start with 65536 chunks (up to 256 GiB)
-    let downloaded_chunks = Arc::new(Mutex::new(HashSet::with_capacity(1024 * 64)));
+    let backup_group = params
+        .target
+        .store
+        .backup_group(target_ns.clone(), group.clone());
+    if let Some(info) = backup_group.last_backup(true).unwrap_or(None) {
+        let mut reusable_chunks = encountered_chunks.lock().unwrap();
+        if let Err(err) = proxmox_lang::try_block!({
+            let _snapshot_guard = info
+                .backup_dir
+                .lock_shared()
+                .with_context(|| format!("failed locking last backup of group {info:?}"))?;
+
+            let (manifest, _) = info.backup_dir.load_manifest().with_context(|| {
+                format!("failed loading manifest of last backup of group {info:?}")
+            })?;
+
+            match manifest.verify_state()? {
+                Some(verify_state) if verify_state.state == VerifyState::Failed => (),
+                _ => {
+                    for file in manifest.files() {
+                        let index: Box<dyn IndexFile> = match ArchiveType::from_path(&file.filename)
+                        {
+                            Ok(ArchiveType::FixedIndex) => {
+                                let mut path = info.backup_dir.full_path();
+                                path.push(&file.filename);
+                                let index =
+                                    params.target.store.open_fixed_reader(&path).with_context(
+                                        || format!("failed loading fixed index {path:?}"),
+                                    )?;
+                                Box::new(index)
+                            }
+                            Ok(ArchiveType::DynamicIndex) => {
+                                let mut path = info.backup_dir.full_path();
+                                path.push(&file.filename);
+                                let index = params
+                                    .target
+                                    .store
+                                    .open_dynamic_reader(&path)
+                                    .with_context(|| {
+                                        format!("failed loading dynamic index {path:?}")
+                                    })?;
+                                Box::new(index)
+                            }
+                            _ => continue,
+                        };
+
+                        for pos in 0..index.index_count() {
+                            let chunk_info = index.chunk_info(pos).unwrap();
+                            reusable_chunks.mark_reusable(&chunk_info.digest);
+                        }
+                    }
+                }
+            }
+            Ok::<(), Error>(())
+        }) {
+            warn!("Failed to collect reusable chunk from last backup: {err:#?}");
+        }
+    }
 
     progress.group_snapshots = list.len() as u64;
 
@@ -643,7 +791,7 @@ async fn pull_group(
             params,
             reader,
             &to_snapshot,
-            downloaded_chunks.clone(),
+            encountered_chunks.clone(),
             corrupt,
         )
         .await;
@@ -829,6 +977,9 @@ pub(crate) async fn pull_store(mut params: PullParameters) -> Result<SyncStats,
     let mut synced_ns = HashSet::with_capacity(namespaces.len());
     let mut sync_stats = SyncStats::default();
 
+    // start with 65536 chunks (up to 256 GiB)
+    let encountered_chunks = Arc::new(Mutex::new(EncounteredChunks::with_capacity(1024 * 64)));
+
     for namespace in namespaces {
         let source_store_ns_str = print_store_and_ns(params.source.get_store(), &namespace);
 
@@ -850,7 +1001,7 @@ pub(crate) async fn pull_store(mut params: PullParameters) -> Result<SyncStats,
             }
         }
 
-        match pull_ns(&namespace, &mut params).await {
+        match pull_ns(&namespace, &mut params, encountered_chunks.clone()).await {
             Ok((ns_progress, ns_sync_stats, ns_errors)) => {
                 errors |= ns_errors;
 
@@ -905,9 +1056,10 @@ pub(crate) async fn pull_store(mut params: PullParameters) -> Result<SyncStats,
 /// Permission checks:
 /// - remote namespaces are filtered by remote
 /// - owner check for vanished groups done here
-pub(crate) async fn pull_ns(
+async fn pull_ns(
     namespace: &BackupNamespace,
     params: &mut PullParameters,
+    encountered_chunks: Arc<Mutex<EncounteredChunks>>,
 ) -> Result<(StoreProgress, SyncStats, bool), Error> {
     let list: Vec<BackupGroup> = params.source.list_groups(namespace, &params.owner).await?;
 
@@ -966,7 +1118,16 @@ pub(crate) async fn pull_ns(
             );
             errors = true; // do not stop here, instead continue
         } else {
-            match pull_group(params, namespace, &group, &mut progress).await {
+            encountered_chunks.lock().unwrap().clear();
+            match pull_group(
+                params,
+                namespace,
+                &group,
+                &mut progress,
+                encountered_chunks.clone(),
+            )
+            .await
+            {
                 Ok(stats) => sync_stats.add(stats),
                 Err(err) => {
                     info!("sync group {} failed - {err:#}", &group);
@@ -1030,3 +1191,66 @@ pub(crate) async fn pull_ns(
 
     Ok((progress, sync_stats, errors))
 }
+
+/// Store the state of encountered chunks, tracking if they can be reused for the
+/// index file currently being pulled and if the chunk has already been touched
+/// during this sync.
+struct EncounteredChunks {
+    // key: digest, value: (reusable, touched)
+    chunk_set: HashMap<[u8; 32], (bool, bool)>,
+}
+
+impl EncounteredChunks {
+    /// Create a new instance with preallocated capacity
+    fn with_capacity(capacity: usize) -> Self {
+        Self {
+            chunk_set: HashMap::with_capacity(capacity),
+        }
+    }
+
+    /// Check if the current state allows to reuse this chunk and if so,
+    /// if the chunk has already been touched.
+    fn check_reusable(&self, digest: &[u8; 32]) -> Option<bool> {
+        if let Some((reusable, touched)) = self.chunk_set.get(digest) {
+            if !reusable {
+                None
+            } else {
+                Some(*touched)
+            }
+        } else {
+            None
+        }
+    }
+
+    /// Mark chunk as reusable, inserting it as un-touched if not present
+    fn mark_reusable(&mut self, digest: &[u8; 32]) {
+        match self.chunk_set.entry(*digest) {
+            Entry::Occupied(mut occupied) => {
+                let (reusable, _touched) = occupied.get_mut();
+                *reusable = true;
+            }
+            Entry::Vacant(vacant) => {
+                vacant.insert((true, false));
+            }
+        }
+    }
+
+    /// Mark chunk as touched during this sync, inserting it as not reusable
+    /// but touched if not present.
+    fn mark_touched(&mut self, digest: &[u8; 32]) {
+        match self.chunk_set.entry(*digest) {
+            Entry::Occupied(mut occupied) => {
+                let (_reusable, touched) = occupied.get_mut();
+                *touched = true;
+            }
+            Entry::Vacant(vacant) => {
+                vacant.insert((false, true));
+            }
+        }
+    }
+
+    /// Clear all entries
+    fn clear(&mut self) {
+        self.chunk_set.clear();
+    }
+}
