@@ -9,7 +9,7 @@ use std::sync::Arc;
 use std::task::{Context, Poll};
 use std::{io, mem};
 
-use anyhow::{bail, format_err, Error};
+use anyhow::{Error, bail, format_err};
 use futures::ready;
 use futures::stream::{FusedStream, Stream};
 
@@ -328,6 +328,32 @@ impl FuseData {
             }));
     }
 
+    extern "C" fn rename(
+        request: sys::Request,
+        parent: u64,
+        name: sys::StrPtr,
+        new_parent: u64,
+        new_name: sys::StrPtr,
+        flags: libc::c_int,
+    ) {
+        let fuse_data = unsafe { &*(sys::fuse_req_userdata(request) as *mut FuseData) };
+        let name = unsafe { CStr::from_ptr(name) };
+        let name = OsStr::from_bytes(name.to_bytes()).to_owned();
+        let new_name = unsafe { CStr::from_ptr(new_name) };
+        let new_name = OsStr::from_bytes(new_name.to_bytes()).to_owned();
+        fuse_data
+            .pending_requests
+            .borrow_mut()
+            .push_back(Request::Rename(requests::Rename {
+                request: RequestGuard::from_raw(request),
+                parent,
+                name,
+                new_parent,
+                new_name,
+                flags,
+            }));
+    }
+
     extern "C" fn write(
         request: sys::Request,
         inode: u64,
@@ -533,6 +559,12 @@ impl FuseSessionBuilder {
         self
     }
 
+    /// Enable `Rename` requests.
+    pub fn enable_rename(mut self) -> Self {
+        self.operations.rename = Some(FuseData::rename);
+        self
+    }
+
     /// Enable `Read` requests.
     pub fn enable_read(mut self) -> Self {
         self.operations.read = Some(FuseData::read);
