@@ -8,6 +8,10 @@ use anyhow::{bail, format_err, Error};
 use const_format::concatcp;
 use serde::{Deserialize, Serialize};
 
+#[cfg(feature = "enum-fallback")]
+use proxmox_fixed_string::FixedString;
+
+use proxmox_human_byte::HumanByte;
 use proxmox_schema::{
     api, const_regex, ApiStringFormat, ApiType, ArraySchema, EnumEntry, IntegerSchema, ReturnType,
     Schema, StringSchema, Updater, UpdaterType,
@@ -18,7 +22,8 @@ use crate::{
     BACKUP_ID_RE, BACKUP_NS_RE, BACKUP_TIME_RE, BACKUP_TYPE_RE, DATASTORE_NOTIFY_STRING_SCHEMA,
     GC_SCHEDULE_SCHEMA, GROUP_OR_SNAPSHOT_PATH_REGEX_STR, PROXMOX_SAFE_ID_FORMAT,
     PROXMOX_SAFE_ID_REGEX_STR, PRUNE_SCHEDULE_SCHEMA, SHA256_HEX_REGEX, SINGLE_LINE_COMMENT_SCHEMA,
-    SNAPSHOT_PATH_REGEX_STR, UPID,
+    SNAPSHOT_PATH_REGEX_STR, UPID, VERIFY_JOB_READ_THREADS_SCHEMA,
+    VERIFY_JOB_VERIFY_THREADS_SCHEMA,
 };
 
 const_regex! {
@@ -92,14 +97,14 @@ pub const BACKUP_NAMESPACE_SCHEMA: Schema = StringSchema::new("Namespace.")
 pub const NS_MAX_DEPTH_SCHEMA: Schema =
     IntegerSchema::new("How many levels of namespaces should be operated on (0 == no recursion)")
         .minimum(0)
-        .maximum(MAX_NAMESPACE_DEPTH as isize)
-        .default(MAX_NAMESPACE_DEPTH as isize)
+        .maximum(MAX_NAMESPACE_DEPTH as i64)
+        .default(MAX_NAMESPACE_DEPTH as i64)
         .schema();
 
 pub const NS_MAX_DEPTH_REDUCED_SCHEMA: Schema =
 IntegerSchema::new("How many levels of namespaces should be operated on (0 == no recursion, empty == automatic full recursion, namespace depths reduce maximum allowed value)")
     .minimum(0)
-    .maximum(MAX_NAMESPACE_DEPTH as isize)
+    .maximum(MAX_NAMESPACE_DEPTH as i64)
     .schema();
 
 pub const DATASTORE_SCHEMA: Schema = StringSchema::new("Datastore name.")
@@ -176,6 +181,9 @@ pub enum ChunkOrder {
     /// Iterate chunks in inode order
     #[default]
     Inode,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 #[api]
@@ -185,11 +193,14 @@ pub enum ChunkOrder {
 pub enum DataStoreMountStatus {
     /// Removable datastore is currently mounted correctly.
     Mounted,
-    /// Removable datastore is currebtly not mounted.
+    /// Removable datastore is currently not mounted.
     NotMounted,
     /// Datastore is not removable, so there is no mount status.
     #[default]
     NonRemovable,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 #[api]
@@ -221,6 +232,9 @@ pub enum DatastoreFSyncLevel {
     /// and consistency.
     #[default]
     Filesystem,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 pub const GC_ATIME_CUTOFF_SCHEMA: Schema = IntegerSchema::new(
@@ -261,6 +275,14 @@ pub const GC_CACHE_CAPACITY_SCHEMA: Schema =
             schema: GC_CACHE_CAPACITY_SCHEMA,
             optional: true,
         },
+        "default-verification-workers": {
+            schema: VERIFY_JOB_VERIFY_THREADS_SCHEMA,
+            optional: true,
+        },
+        "default-verification-readers": {
+            schema: VERIFY_JOB_READ_THREADS_SCHEMA,
+            optional: true,
+        },
     },
 )]
 #[derive(Serialize, Deserialize, Default)]
@@ -278,6 +300,10 @@ pub struct DatastoreTuning {
     pub gc_atime_cutoff: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub gc_cache_capacity: Option<usize>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub default_verification_workers: Option<usize>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub default_verification_readers: Option<usize>,
 }
 
 pub const DATASTORE_TUNING_STRING_SCHEMA: Schema = StringSchema::new("Datastore tuning options")
@@ -286,6 +312,113 @@ pub const DATASTORE_TUNING_STRING_SCHEMA: Schema = StringSchema::new("Datastore
     ))
     .schema();
 
+#[api]
+#[derive(Copy, Clone, Default, Deserialize, Serialize, Updater, PartialEq)]
+#[serde(rename_all = "kebab-case")]
+/// Datastore backend type
+pub enum DatastoreBackendType {
+    /// Local filesystem
+    #[default]
+    Filesystem,
+    /// S3 object store
+    S3,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
+}
+serde_plain::derive_display_from_serialize!(DatastoreBackendType);
+serde_plain::derive_fromstr_from_deserialize!(DatastoreBackendType);
+
+#[api(
+    properties: {
+        type: {
+            type: DatastoreBackendType,
+            optional: true,
+        },
+        client: {
+            schema: proxmox_s3_client::S3_CLIENT_ID_SCHEMA,
+            optional: true,
+        },
+        bucket: {
+            schema: proxmox_s3_client::S3_BUCKET_NAME_SCHEMA,
+            optional: true,
+        },
+        "max-cache-size": {
+            type: HumanByte,
+            optional: true,
+        }
+    },
+    default_key: "type",
+)]
+#[derive(Default, Deserialize, Serialize)]
+#[serde(rename_all = "kebab-case")]
+/// Datastore backend config
+pub struct DatastoreBackendConfig {
+    /// backend type
+    #[serde(rename = "type")]
+    pub ty: Option<DatastoreBackendType>,
+    /// s3 client id
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub client: Option<String>,
+    /// s3 bucket name
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub bucket: Option<String>,
+    /// maximum cache size for local datastore LRU cache
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub max_cache_size: Option<HumanByte>,
+}
+
+pub const DATASTORE_BACKEND_CONFIG_STRING_SCHEMA: Schema =
+    StringSchema::new("Datastore backend config")
+        .format(&ApiStringFormat::VerifyFn(verify_datastore_backend_config))
+        .type_text("<backend-config>")
+        .schema();
+
+fn verify_datastore_backend_config(input: &str) -> Result<(), Error> {
+    DatastoreBackendConfig::from_str(input).map(|_| ())
+}
+
+impl FromStr for DatastoreBackendConfig {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let backend_config: DatastoreBackendConfig =
+            proxmox_schema::property_string::parse_with_schema(
+                s,
+                &DatastoreBackendConfig::API_SCHEMA,
+            )?;
+        let backend_type = backend_config.ty.unwrap_or_default();
+        match backend_type {
+            DatastoreBackendType::Filesystem => {
+                if backend_config.client.is_some() {
+                    bail!("additional option client, not allowed for backend type filesystem");
+                }
+                if backend_config.bucket.is_some() {
+                    bail!("additional option bucket, not allowed for backend type filesystem");
+                }
+                if backend_config.max_cache_size.is_some() {
+                    bail!(
+                        "additional option max-cache-size, not allowed for backend type filesystem"
+                    );
+                }
+            }
+            DatastoreBackendType::S3 => {
+                if backend_config.client.is_none() {
+                    bail!("missing option client, required for backend type s3");
+                }
+                if backend_config.bucket.is_none() {
+                    bail!("missing option bucket, required for backend type s3");
+                }
+            }
+            #[cfg(feature = "enum-fallback")]
+            DatastoreBackendType::UnknownEnumValue(s) => {
+                bail!("unknown backend type: {s}");
+            }
+        }
+        Ok(backend_config)
+    }
+}
+
 #[api(
     properties: {
         name: {
@@ -336,7 +469,11 @@ pub const DATASTORE_TUNING_STRING_SCHEMA: Schema = StringSchema::new("Datastore
             optional: true,
             format: &proxmox_schema::api_types::UUID_FORMAT,
             type: String,
-        }
+        },
+        backend: {
+            schema: DATASTORE_BACKEND_CONFIG_STRING_SCHEMA,
+            optional: true,
+        },
     }
 )]
 #[derive(Serialize, Deserialize, Updater, Clone, PartialEq)]
@@ -389,6 +526,11 @@ pub struct DataStoreConfig {
     #[updater(skip)]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub backing_device: Option<String>,
+
+    /// Backend configuration for datastore
+    #[updater(skip)]
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub backend: Option<String>,
 }
 
 #[api]
@@ -402,10 +544,13 @@ pub struct DataStoreConfig {
 pub enum NotificationMode {
     /// Send notifications via the system's sendmail command to the user
     /// configured in `notify-user`
-    #[default]
     LegacySendmail,
     /// Emit notification events to the notification system
+    #[default]
     NotificationSystem,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 impl DataStoreConfig {
@@ -424,6 +569,7 @@ impl DataStoreConfig {
             tuning: None,
             maintenance_mode: None,
             backing_device: None,
+            backend: None,
         }
     }
 
@@ -456,6 +602,7 @@ impl DataStoreConfig {
             Some(MaintenanceType::Unmount) => {
                 /* used to reset it after failed unmount, or alternative for aborting unmount task */
             }
+            Some(MaintenanceType::S3Refresh) => { /* used to reset state after refresh finished */ }
             Some(MaintenanceType::Delete) => {
                 match new_type {
                     Some(MaintenanceType::Delete) => { /* allow to delete a deleted storage */ }
@@ -464,6 +611,10 @@ impl DataStoreConfig {
                     }
                 }
             }
+            #[cfg(feature = "enum-fallback")]
+            Some(MaintenanceType::UnknownEnumValue(s)) => {
+                bail!("unknown maintenance type: {s}")
+            }
             None => { /* always OK  */ }
         }
 
@@ -585,6 +736,9 @@ pub enum VerifyState {
     Ok,
     /// Verification reported one or more errors
     Failed,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 #[api(
@@ -801,7 +955,7 @@ impl BackupNamespace {
 
     /// Return an adapter which [`fmt::Display`]s as a path with `"ns/"` prefixes in front of every
     /// component.
-    pub fn display_as_path(&self) -> BackupNamespacePath {
+    pub fn display_as_path(&self) -> BackupNamespacePath<'_> {
         BackupNamespacePath(self)
     }
 
@@ -946,6 +1100,9 @@ pub enum BackupType {
 
     /// "Host" backups.
     Host,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
     // NOTE: if you add new types, don't forget to adapt the iter below!
 }
 
@@ -955,6 +1112,8 @@ impl BackupType {
             BackupType::Vm => "vm",
             BackupType::Ct => "ct",
             BackupType::Host => "host",
+            #[cfg(feature = "enum-fallback")]
+            BackupType::UnknownEnumValue(_) => "unknown backup type",
         }
     }
 
@@ -964,6 +1123,8 @@ impl BackupType {
             BackupType::Ct => 0,
             BackupType::Host => 1,
             BackupType::Vm => 2,
+            #[cfg(feature = "enum-fallback")]
+            BackupType::UnknownEnumValue(_) => u8::MAX,
         }
     }
 
@@ -1426,6 +1587,16 @@ pub struct TypeCounts {
     pub snapshots: u64,
 }
 
+#[api()]
+#[derive(Clone, Debug, Default, Serialize, Deserialize, PartialEq)]
+/// Garbage collection cache statistics
+pub struct GarbageCollectionCacheStats {
+    /// Number of atime update cache hits
+    pub hits: usize,
+    /// Number of atime update cache misses
+    pub misses: usize,
+}
+
 #[api(
     properties: {
         "upid": {
@@ -1459,6 +1630,9 @@ pub struct GarbageCollectionStatus {
     pub removed_bad: usize,
     /// Number of chunks still marked as .bad after garbage collection.
     pub still_bad: usize,
+    /// Statistics of atime update cache
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub cache_stats: Option<GarbageCollectionCacheStats>,
 }
 
 #[api(
@@ -1702,9 +1876,9 @@ pub fn print_ns_and_snapshot(ns: &BackupNamespace, dir: &BackupDir) -> String {
 /// Prints a Datastore name and [`BackupNamespace`] for logs/errors.
 pub fn print_store_and_ns(store: &str, ns: &BackupNamespace) -> String {
     if ns.is_root() {
-        format!("datastore '{}', root namespace", store)
+        format!("datastore '{store}', root namespace")
     } else {
-        format!("datastore '{}', namespace '{}'", store, ns)
+        format!("datastore '{store}', namespace '{ns}'")
     }
 }
 
