@@ -5,6 +5,9 @@ use const_format::concatcp;
 use regex::Regex;
 use serde::{Deserialize, Serialize};
 
+#[cfg(feature = "enum-fallback")]
+use proxmox_fixed_string::FixedString;
+
 use proxmox_schema::*;
 
 use crate::{
@@ -64,6 +67,27 @@ pub const REMOVE_VANISHED_BACKUPS_SCHEMA: Schema = BooleanSchema::new(
 .default(false)
 .schema();
 
+const fn threads_schema(description: &'static str, default: i64) -> Schema {
+    IntegerSchema::new(description)
+        .minimum(1)
+        .maximum(32)
+        .default(default)
+        .schema()
+}
+
+pub const TAPE_JOB_THREADS_SCHEMA: Schema =
+    threads_schema("The number of threads to use for the tape backup job.", 1);
+
+pub const VERIFY_JOB_READ_THREADS_SCHEMA: Schema = threads_schema(
+    "The number of threads to use for reading chunks in verify job.",
+    1,
+);
+
+pub const VERIFY_JOB_VERIFY_THREADS_SCHEMA: Schema = threads_schema(
+    "The number of threads to use for verifying chunks in verify job.",
+    4,
+);
+
 #[api(
     properties: {
         "next-run": {
@@ -113,6 +137,9 @@ pub enum Notify {
     Always,
     /// Send notifications for failed jobs only
     Error,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 #[api(
@@ -203,6 +230,14 @@ pub const VERIFICATION_OUTDATED_AFTER_SCHEMA: Schema =
             optional: true,
             schema: crate::NS_MAX_DEPTH_SCHEMA,
         },
+        "read-threads": {
+            schema: VERIFY_JOB_READ_THREADS_SCHEMA,
+            optional: true,
+        },
+        "verify-threads": {
+            schema: VERIFY_JOB_VERIFY_THREADS_SCHEMA,
+            optional: true,
+        },
     }
 )]
 #[derive(Serialize, Deserialize, Updater, Clone, PartialEq)]
@@ -233,6 +268,12 @@ pub struct VerificationJobConfig {
     /// how deep the verify should go from the `ns` level downwards. Passing 0 verifies only the
     /// snapshots on the same level as the passed `ns`, or the datastore root if none.
     pub max_depth: Option<usize>,
+    /// The number of read threads to use for the job.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub read_threads: Option<usize>,
+    /// The number of verification threads to use for the job.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub verify_threads: Option<usize>,
 }
 
 impl VerificationJobConfig {
@@ -307,11 +348,8 @@ pub struct VerificationJobStatus {
             optional: true,
         },
         "worker-threads": {
-            type: Integer,
+            schema: TAPE_JOB_THREADS_SCHEMA,
             optional: true,
-            minimum: 1,
-            maximum: 32,
-            default: 1,
         },
     }
 )]
@@ -339,7 +377,6 @@ pub struct TapeBackupJobSetup {
     pub ns: Option<BackupNamespace>,
     #[serde(skip_serializing_if = "Option::is_none", default)]
     pub max_depth: Option<usize>,
-    /// Set the number of worker threads to use for the job
     #[serde(skip_serializing_if = "Option::is_none")]
     pub worker_threads: Option<u64>,
 }
@@ -439,8 +476,8 @@ impl std::str::FromStr for FilterType {
 impl std::fmt::Display for FilterType {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            FilterType::BackupType(backup_type) => write!(f, "type:{}", backup_type),
-            FilterType::Group(backup_group) => write!(f, "group:{}", backup_group),
+            FilterType::BackupType(backup_type) => write!(f, "type:{backup_type}"),
+            FilterType::Group(backup_group) => write!(f, "group:{backup_group}"),
             FilterType::Regex(regex) => write!(f, "regex:{}", regex.as_str()),
         }
     }
@@ -518,6 +555,9 @@ pub enum SyncDirection {
     Pull,
     /// Sync direction push
     Push,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 impl std::fmt::Display for SyncDirection {
@@ -525,6 +565,10 @@ impl std::fmt::Display for SyncDirection {
         match self {
             SyncDirection::Pull => f.write_str("pull"),
             SyncDirection::Push => f.write_str("push"),
+            #[cfg(feature = "enum-fallback")]
+            SyncDirection::UnknownEnumValue(s) => {
+                write!(f, "unknown sync direction: {s}")
+            }
         }
     }
 }
@@ -536,6 +580,11 @@ pub const SYNC_ENCRYPTED_ONLY_SCHEMA: Schema =
     BooleanSchema::new("Only synchronize encrypted backup snapshots, exclude others.").schema();
 pub const SYNC_VERIFIED_ONLY_SCHEMA: Schema =
     BooleanSchema::new("Only synchronize verified backup snapshots, exclude others.").schema();
+pub const RUN_SYNC_ON_MOUNT_SCHEMA: Schema =
+    BooleanSchema::new("Run this job when a relevant datastore is mounted.").schema();
+pub const UNMOUNT_ON_SYNC_DONE_SCHEMA: Schema =
+    BooleanSchema::new("Unmount involved removable datastore after the sync job finishes. Requires 'run-on-mount' to be enabled.")
+        .schema();
 
 #[api(
     properties: {
@@ -603,6 +652,14 @@ pub const SYNC_VERIFIED_ONLY_SCHEMA: Schema =
             schema: SYNC_VERIFIED_ONLY_SCHEMA,
             optional: true,
         },
+        "run-on-mount": {
+            schema: RUN_SYNC_ON_MOUNT_SCHEMA,
+            optional: true,
+        },
+        "unmount-on-done": {
+            schema: UNMOUNT_ON_SYNC_DONE_SCHEMA,
+            optional: true,
+        },
         "sync-direction": {
             type: SyncDirection,
             optional: true,
@@ -647,6 +704,10 @@ pub struct SyncJobConfig {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub verified_only: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
+    pub run_on_mount: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub unmount_on_done: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub sync_direction: Option<SyncDirection>,
 }
 
