@@ -1,5 +1,8 @@
 use ::serde::{Deserialize, Serialize};
 
+#[cfg(feature = "enum-fallback")]
+use proxmox_fixed_string::FixedString;
+
 use proxmox_schema::api;
 
 #[api()]
@@ -27,6 +30,9 @@ pub enum DeviceKind {
     Changer,
     /// Normal SCSI tape device
     Tape,
+    #[cfg(feature = "enum-fallback")]
+    #[serde(untagged)]
+    UnknownEnumValue(FixedString),
 }
 
 #[api(
