@@ -1,15 +1,18 @@
-use std::collections::{BTreeMap, BTreeSet, HashMap};
+#[cfg(feature = "impl")]
+use std::collections::BTreeSet;
+use std::collections::{BTreeMap, HashMap};
+#[cfg(feature = "impl")]
 use std::io::Write;
+#[cfg(feature = "impl")]
 use std::path::Path;
-use std::sync::{Arc, OnceLock, RwLock};
 
 use anyhow::{bail, Error};
 
 use proxmox_auth_api::types::{Authid, Userid};
+#[cfg(feature = "impl")]
 use proxmox_config_digest::ConfigDigest;
-use proxmox_product_config::{open_api_lockfile, replace_privileged_config, ApiLockGuard};
 
-use crate::init::{access_conf, acl_config, acl_config_lock};
+use crate::init::access_conf;
 
 pub fn split_acl_path(path: &str) -> Vec<&str> {
     let items = path.split('/');
@@ -27,7 +30,7 @@ pub fn split_acl_path(path: &str) -> Vec<&str> {
 }
 
 /// Tree representing a parsed acl.cfg
-#[derive(Default)]
+#[derive(Default, Debug, PartialEq)]
 pub struct AclTree {
     /// Root node of the tree.
     ///
@@ -37,7 +40,7 @@ pub struct AclTree {
 }
 
 /// Node representing ACLs for a certain ACL path.
-#[derive(Default)]
+#[derive(Default, Debug, PartialEq)]
 pub struct AclTreeNode {
     /// `User` or `Token` ACLs for this node.
     pub users: HashMap<Authid, HashMap<String, bool>>,
@@ -302,110 +305,6 @@ impl AclTree {
         node.insert_user_role(auth_id.to_owned(), role.to_string(), propagate);
     }
 
-    fn write_node_config(node: &AclTreeNode, path: &str, w: &mut dyn Write) -> Result<(), Error> {
-        let mut role_ug_map0: HashMap<_, BTreeSet<_>> = HashMap::new();
-        let mut role_ug_map1: HashMap<_, BTreeSet<_>> = HashMap::new();
-
-        for (auth_id, roles) in &node.users {
-            // no need to save, because root is always 'Administrator'
-            if !auth_id.is_token() && auth_id.user() == "root@pam" {
-                continue;
-            }
-            for (role, propagate) in roles {
-                let role = role.as_str();
-                let auth_id = auth_id.to_string();
-                if *propagate {
-                    role_ug_map1.entry(role).or_default().insert(auth_id);
-                } else {
-                    role_ug_map0.entry(role).or_default().insert(auth_id);
-                }
-            }
-        }
-
-        for (group, roles) in &node.groups {
-            for (role, propagate) in roles {
-                let group = format!("@{}", group);
-                if *propagate {
-                    role_ug_map1.entry(role).or_default().insert(group);
-                } else {
-                    role_ug_map0.entry(role).or_default().insert(group);
-                }
-            }
-        }
-
-        fn group_by_property_list(
-            item_property_map: &HashMap<&str, BTreeSet<String>>,
-        ) -> BTreeMap<String, BTreeSet<String>> {
-            let mut result_map: BTreeMap<_, BTreeSet<_>> = BTreeMap::new();
-            for (item, property_map) in item_property_map {
-                let item_list = property_map.iter().fold(String::new(), |mut acc, v| {
-                    if !acc.is_empty() {
-                        acc.push(',');
-                    }
-                    acc.push_str(v);
-                    acc
-                });
-                result_map
-                    .entry(item_list)
-                    .or_default()
-                    .insert(item.to_string());
-            }
-            result_map
-        }
-
-        let uglist_role_map0 = group_by_property_list(&role_ug_map0);
-        let uglist_role_map1 = group_by_property_list(&role_ug_map1);
-
-        fn role_list(roles: &BTreeSet<String>) -> String {
-            if let Some(no_access) = access_conf().role_no_access() {
-                if roles.contains(no_access) {
-                    return String::from(no_access);
-                }
-            }
-
-            roles.iter().fold(String::new(), |mut acc, v| {
-                if !acc.is_empty() {
-                    acc.push(',');
-                }
-                acc.push_str(v);
-                acc
-            })
-        }
-
-        for (uglist, roles) in &uglist_role_map0 {
-            let role_list = role_list(roles);
-            writeln!(
-                w,
-                "acl:0:{}:{}:{}",
-                if path.is_empty() { "/" } else { path },
-                uglist,
-                role_list
-            )?;
-        }
-
-        for (uglist, roles) in &uglist_role_map1 {
-            let role_list = role_list(roles);
-            writeln!(
-                w,
-                "acl:1:{}:{}:{}",
-                if path.is_empty() { "/" } else { path },
-                uglist,
-                role_list
-            )?;
-        }
-
-        for (name, child) in node.children.iter() {
-            let child_path = format!("{}/{}", path, name);
-            Self::write_node_config(child, &child_path, w)?;
-        }
-
-        Ok(())
-    }
-
-    fn write_config(&self, w: &mut dyn Write) -> Result<(), Error> {
-        Self::write_node_config(&self.root, "", w)
-    }
-
     fn parse_acl_line(&mut self, line: &str) -> Result<(), Error> {
         let items: Vec<&str> = line.split(':').collect();
 
@@ -436,7 +335,7 @@ impl AclTree {
         for user_or_group in &uglist {
             for role in &rolelist {
                 if !access_conf().roles().contains_key(role) {
-                    bail!("unknown role '{}'", role);
+                    bail!("unknown role '{role}'");
                 }
                 if let Some(group) = user_or_group.strip_prefix('@') {
                     node.insert_group_role(group.to_string(), role.to_string(), propagate);
@@ -449,40 +348,6 @@ impl AclTree {
         Ok(())
     }
 
-    fn load(filename: &Path) -> Result<(Self, ConfigDigest), Error> {
-        let mut tree = Self::new();
-
-        let raw = match std::fs::read_to_string(filename) {
-            Ok(v) => v,
-            Err(err) => {
-                if err.kind() == std::io::ErrorKind::NotFound {
-                    String::new()
-                } else {
-                    bail!("unable to read acl config {:?} - {}", filename, err);
-                }
-            }
-        };
-
-        let digest = ConfigDigest::from_slice(raw.as_bytes());
-
-        for (linenr, line) in raw.lines().enumerate() {
-            let line = line.trim();
-            if line.is_empty() {
-                continue;
-            }
-            if let Err(err) = tree.parse_acl_line(line) {
-                bail!(
-                    "unable to parse acl config {:?}, line {} - {}",
-                    filename,
-                    linenr + 1,
-                    err
-                );
-            }
-        }
-
-        Ok((tree, digest))
-    }
-
     /// This is used for testing
     pub fn from_raw(raw: &str) -> Result<Self, Error> {
         let mut tree = Self::new();
@@ -493,9 +358,8 @@ impl AclTree {
             }
             if let Err(err) = tree.parse_acl_line(line) {
                 bail!(
-                    "unable to parse acl config data, line {} - {}",
+                    "unable to parse acl config data, line {} - {err}",
                     linenr + 1,
-                    err
                 );
             }
         }
@@ -550,88 +414,349 @@ impl AclTree {
 
         Ok(res)
     }
-}
 
-/// Get exclusive lock
-pub fn lock_config() -> Result<ApiLockGuard, Error> {
-    open_api_lockfile(acl_config_lock(), None, true)
-}
-
-/// Reads the [`AclTree`] from `acl.cfg` in the configuration directory.
-pub fn config() -> Result<(AclTree, ConfigDigest), Error> {
-    let path = acl_config();
-    AclTree::load(&path)
-}
-
-/// Returns a cached [`AclTree`] or a fresh copy read directly from `acl.cfg` in the configuration
-/// directory.
-///
-/// Since the AclTree is used for every API request's permission check, this caching mechanism
-/// allows to skip reading and parsing the file again if it is unchanged.
-pub fn cached_config() -> Result<Arc<AclTree>, Error> {
-    struct ConfigCache {
-        data: Option<Arc<AclTree>>,
-        last_mtime: i64,
-        last_mtime_nsec: i64,
+    pub fn check_privs(
+        &self,
+        auth_id: &Authid,
+        path: &[&str],
+        required_privs: u64,
+        partial: bool,
+    ) -> Result<(), Error> {
+        let privs = self.lookup_privs(auth_id, path);
+        let allowed = if partial {
+            (privs & required_privs) != 0
+        } else {
+            (privs & required_privs) == required_privs
+        };
+        if !allowed {
+            // printing the path doesn't leak any information as long as we
+            // always check privilege before resource existence
+            let priv_names = privs_to_priv_names(required_privs);
+            let priv_names = if partial {
+                priv_names.join("|")
+            } else {
+                priv_names.join("&")
+            };
+            bail!(
+                "missing permissions '{priv_names}' on '/{}'",
+                path.join("/")
+            );
+        }
+        Ok(())
     }
 
-    static CACHED_CONFIG: OnceLock<RwLock<ConfigCache>> = OnceLock::new();
-    let cached_conf = CACHED_CONFIG.get_or_init(|| {
-        RwLock::new(ConfigCache {
-            data: None,
-            last_mtime: 0,
-            last_mtime_nsec: 0,
-        })
-    });
+    pub fn lookup_privs(&self, auth_id: &Authid, path: &[&str]) -> u64 {
+        let (privs, _) = self.lookup_privs_details(auth_id, path);
+        privs
+    }
 
-    let conf = acl_config();
-    let stat = match nix::sys::stat::stat(&conf) {
-        Ok(stat) => Some(stat),
-        Err(nix::errno::Errno::ENOENT) => None,
-        Err(err) => bail!("unable to stat '{}' - {err}", conf.display()),
-    };
-
-    {
-        // limit scope
-        let cache = cached_conf.read().unwrap();
-        if let Some(ref config) = cache.data {
-            if let Some(stat) = stat {
-                if stat.st_mtime == cache.last_mtime && stat.st_mtime_nsec == cache.last_mtime_nsec
-                {
-                    return Ok(config.clone());
+    pub fn lookup_privs_details(&self, auth_id: &Authid, path: &[&str]) -> (u64, u64) {
+        if access_conf().is_superuser(auth_id) {
+            let acm_config = access_conf();
+            if let Some(admin) = acm_config.role_admin() {
+                if let Some((admin, _)) = acm_config.roles().get(admin) {
+                    return (*admin, *admin);
                 }
-            } else if cache.last_mtime == 0 && cache.last_mtime_nsec == 0 {
-                return Ok(config.clone());
             }
         }
+
+        let roles = self.roles(auth_id, path);
+        let mut privs: u64 = 0;
+        let mut propagated_privs: u64 = 0;
+        for (role, propagate) in roles {
+            if let Some((role_privs, _)) = access_conf().roles().get(role.as_str()) {
+                if propagate {
+                    propagated_privs |= role_privs;
+                }
+                privs |= role_privs;
+            }
+        }
+
+        if auth_id.is_token() {
+            // limit privs to that of owning user
+            let user_auth_id = Authid::from(auth_id.user().clone());
+            let (owner_privs, owner_propagated_privs) =
+                self.lookup_privs_details(&user_auth_id, path);
+            privs &= owner_privs;
+            propagated_privs &= owner_propagated_privs;
+        }
+
+        (privs, propagated_privs)
     }
 
-    let (config, _digest) = config()?;
-    let config = Arc::new(config);
+    /// Checks whether the `auth_id` has any of the privileges `privs` on any object below `path`.
+    pub fn any_privs_below(
+        &self,
+        auth_id: &Authid,
+        path: &[&str],
+        privs: u64,
+    ) -> Result<bool, Error> {
+        // if the anchor path itself has matching propagated privs, we skip checking children
+        let (_privs, propagated_privs) = self.lookup_privs_details(auth_id, path);
+        if propagated_privs & privs != 0 {
+            return Ok(true);
+        }
 
-    let mut cache = cached_conf.write().unwrap();
-    if let Some(stat) = stat {
-        cache.last_mtime = stat.st_mtime;
-        cache.last_mtime_nsec = stat.st_mtime_nsec;
+        // get all sub-paths with roles defined for `auth_id`
+        let paths = self.get_child_paths(auth_id, path)?;
+
+        for path in paths.iter() {
+            // early return if any sub-path has any of the privs we are looking for
+            if privs & self.lookup_privs(auth_id, &[path.as_str()]) != 0 {
+                return Ok(true);
+            }
+        }
+
+        // no paths or no matching paths
+        Ok(false)
     }
-    cache.data = Some(config.clone());
-
-    Ok(config)
 }
 
-/// Saves an [`AclTree`] to `acl.cfg` in the configuration directory, ensuring proper ownership and
-/// file permissions.
-pub fn save_config(acl: &AclTree) -> Result<(), Error> {
-    let mut raw: Vec<u8> = Vec::new();
-    acl.write_config(&mut raw)?;
+#[cfg(feature = "impl")]
+impl AclTree {
+    fn write_node_config(node: &AclTreeNode, path: &str, w: &mut dyn Write) -> Result<(), Error> {
+        let mut role_ug_map0: HashMap<_, BTreeSet<_>> = HashMap::new();
+        let mut role_ug_map1: HashMap<_, BTreeSet<_>> = HashMap::new();
 
-    let conf = acl_config();
-    replace_privileged_config(conf, &raw)?;
+        for (auth_id, roles) in &node.users {
+            // no need to save, because root is always 'Administrator'
+            if !auth_id.is_token() && auth_id.user() == "root@pam" {
+                continue;
+            }
+            for (role, propagate) in roles {
+                let role = role.as_str();
+                let auth_id = auth_id.to_string();
+                if *propagate {
+                    role_ug_map1.entry(role).or_default().insert(auth_id);
+                } else {
+                    role_ug_map0.entry(role).or_default().insert(auth_id);
+                }
+            }
+        }
 
-    // increase cache generation so we reload it next time we access it
-    access_conf().increment_cache_generation()?;
+        for (group, roles) in &node.groups {
+            for (role, propagate) in roles {
+                let group = format!("@{group}");
+                if *propagate {
+                    role_ug_map1.entry(role).or_default().insert(group);
+                } else {
+                    role_ug_map0.entry(role).or_default().insert(group);
+                }
+            }
+        }
 
-    Ok(())
+        fn group_by_property_list(
+            item_property_map: &HashMap<&str, BTreeSet<String>>,
+        ) -> BTreeMap<String, BTreeSet<String>> {
+            let mut result_map: BTreeMap<_, BTreeSet<_>> = BTreeMap::new();
+            for (item, property_map) in item_property_map {
+                let item_list = property_map.iter().fold(String::new(), |mut acc, v| {
+                    if !acc.is_empty() {
+                        acc.push(',');
+                    }
+                    acc.push_str(v);
+                    acc
+                });
+                result_map
+                    .entry(item_list)
+                    .or_default()
+                    .insert(item.to_string());
+            }
+            result_map
+        }
+
+        let uglist_role_map0 = group_by_property_list(&role_ug_map0);
+        let uglist_role_map1 = group_by_property_list(&role_ug_map1);
+
+        fn role_list(roles: &BTreeSet<String>) -> String {
+            if let Some(no_access) = access_conf().role_no_access() {
+                if roles.contains(no_access) {
+                    return String::from(no_access);
+                }
+            }
+
+            roles.iter().fold(String::new(), |mut acc, v| {
+                if !acc.is_empty() {
+                    acc.push(',');
+                }
+                acc.push_str(v);
+                acc
+            })
+        }
+
+        for (uglist, roles) in &uglist_role_map0 {
+            let role_list = role_list(roles);
+            writeln!(
+                w,
+                "acl:0:{}:{uglist}:{role_list}",
+                if path.is_empty() { "/" } else { path },
+            )?;
+        }
+
+        for (uglist, roles) in &uglist_role_map1 {
+            let role_list = role_list(roles);
+            writeln!(
+                w,
+                "acl:1:{}:{uglist}:{role_list}",
+                if path.is_empty() { "/" } else { path },
+            )?;
+        }
+
+        for (name, child) in node.children.iter() {
+            let child_path = format!("{path}/{name}");
+            Self::write_node_config(child, &child_path, w)?;
+        }
+
+        Ok(())
+    }
+
+    fn write_config(&self, w: &mut dyn Write) -> Result<(), Error> {
+        Self::write_node_config(&self.root, "", w)
+    }
+
+    fn load(filename: &Path) -> Result<(Self, ConfigDigest), Error> {
+        let mut tree = Self::new();
+
+        let raw = match std::fs::read_to_string(filename) {
+            Ok(v) => v,
+            Err(err) => {
+                if err.kind() == std::io::ErrorKind::NotFound {
+                    String::new()
+                } else {
+                    bail!("unable to read acl config {filename:?} - {err}");
+                }
+            }
+        };
+
+        let digest = ConfigDigest::from_slice(raw.as_bytes());
+
+        for (linenr, line) in raw.lines().enumerate() {
+            let line = line.trim();
+            if line.is_empty() {
+                continue;
+            }
+            if let Err(err) = tree.parse_acl_line(line) {
+                bail!(
+                    "unable to parse acl config {filename:?}, line {} - {err}",
+                    linenr + 1
+                );
+            }
+        }
+
+        Ok((tree, digest))
+    }
+}
+
+#[cfg(feature = "impl")]
+pub use impl_feature::{cached_config, config, lock_config, save_config};
+
+#[cfg(feature = "impl")]
+mod impl_feature {
+    use std::sync::{Arc, OnceLock, RwLock};
+
+    use anyhow::{bail, Error};
+
+    use proxmox_config_digest::ConfigDigest;
+    use proxmox_product_config::{open_api_lockfile, replace_privileged_config, ApiLockGuard};
+
+    use crate::acl::AclTree;
+    use crate::init::access_conf;
+    use crate::init::impl_feature::{acl_config, acl_config_lock};
+
+    /// Get exclusive lock
+    pub fn lock_config() -> Result<ApiLockGuard, Error> {
+        open_api_lockfile(acl_config_lock(), None, true)
+    }
+
+    /// Reads the [`AclTree`] from `acl.cfg` in the configuration directory.
+    pub fn config() -> Result<(AclTree, ConfigDigest), Error> {
+        let path = acl_config();
+        AclTree::load(&path)
+    }
+
+    /// Returns a cached [`AclTree`] or a fresh copy read directly from `acl.cfg` in the configuration
+    /// directory.
+    ///
+    /// Since the AclTree is used for every API request's permission check, this caching mechanism
+    /// allows to skip reading and parsing the file again if it is unchanged.
+    pub fn cached_config() -> Result<Arc<AclTree>, Error> {
+        struct ConfigCache {
+            data: Option<Arc<AclTree>>,
+            last_mtime: i64,
+            last_mtime_nsec: i64,
+        }
+
+        static CACHED_CONFIG: OnceLock<RwLock<ConfigCache>> = OnceLock::new();
+        let cached_conf = CACHED_CONFIG.get_or_init(|| {
+            RwLock::new(ConfigCache {
+                data: None,
+                last_mtime: 0,
+                last_mtime_nsec: 0,
+            })
+        });
+
+        let conf = acl_config();
+        let stat = match nix::sys::stat::stat(&conf) {
+            Ok(stat) => Some(stat),
+            Err(nix::errno::Errno::ENOENT) => None,
+            Err(err) => bail!("unable to stat '{}' - {err}", conf.display()),
+        };
+
+        {
+            // limit scope
+            let cache = cached_conf.read().unwrap();
+            if let Some(ref config) = cache.data {
+                if let Some(stat) = stat {
+                    if stat.st_mtime == cache.last_mtime
+                        && stat.st_mtime_nsec == cache.last_mtime_nsec
+                    {
+                        return Ok(config.clone());
+                    }
+                } else if cache.last_mtime == 0 && cache.last_mtime_nsec == 0 {
+                    return Ok(config.clone());
+                }
+            }
+        }
+
+        let (config, _digest) = config()?;
+        let config = Arc::new(config);
+
+        let mut cache = cached_conf.write().unwrap();
+        if let Some(stat) = stat {
+            cache.last_mtime = stat.st_mtime;
+            cache.last_mtime_nsec = stat.st_mtime_nsec;
+        }
+        cache.data = Some(config.clone());
+
+        Ok(config)
+    }
+
+    /// Saves an [`AclTree`] to `acl.cfg` in the configuration directory, ensuring proper ownership and
+    /// file permissions.
+    pub fn save_config(acl: &AclTree) -> Result<(), Error> {
+        let mut raw: Vec<u8> = Vec::new();
+        acl.write_config(&mut raw)?;
+
+        let conf = acl_config();
+        replace_privileged_config(conf, &raw)?;
+
+        // increase cache generation so we reload it next time we access it
+        access_conf().increment_cache_generation()?;
+
+        Ok(())
+    }
+}
+
+fn privs_to_priv_names(privs: u64) -> Vec<&'static str> {
+    access_conf()
+        .privileges()
+        .iter()
+        .fold(Vec::new(), |mut priv_names, (name, value)| {
+            if value & privs != 0 {
+                priv_names.push(name);
+            }
+            priv_names
+        })
 }
 
 #[cfg(test)]
@@ -647,11 +772,11 @@ mod test {
 
     #[derive(Debug)]
     struct TestAcmConfig<'a> {
-        roles: HashMap<&'a str, u64>,
+        roles: HashMap<&'a str, (u64, &'a str)>,
     }
 
     impl AccessControlConfig for TestAcmConfig<'_> {
-        fn roles(&self) -> &HashMap<&str, u64> {
+        fn roles(&self) -> &HashMap<&str, (u64, &str)> {
             &self.roles
         }
 
@@ -672,10 +797,10 @@ mod test {
         static ACL_CONFIG: OnceLock<TestAcmConfig> = OnceLock::new();
         let config = ACL_CONFIG.get_or_init(|| {
             let mut roles = HashMap::new();
-            roles.insert("NoAccess", 0);
-            roles.insert("Admin", u64::MAX);
-            roles.insert("DatastoreBackup", 4);
-            roles.insert("DatastoreReader", 8);
+            roles.insert("NoAccess", (0, "comment"));
+            roles.insert("Admin", (u64::MAX, "comment"));
+            roles.insert("DatastoreBackup", (4, "comment"));
+            roles.insert("DatastoreReader", (8, "comment"));
 
             TestAcmConfig { roles }
         });
@@ -696,12 +821,12 @@ mod test {
 
         assert_eq!(
             roles, expected_roles,
-            "\nat check_roles for '{}' on '{}'",
-            auth_id, path
+            "\nat check_roles for '{auth_id}' on '{path}'"
         );
     }
 
     #[test]
+    #[cfg(feature = "impl")]
     fn test_acl_line_compression() {
         setup_acl_tree_config();
 
@@ -787,6 +912,7 @@ mod test {
     }
 
     #[test]
+    #[cfg(feature = "impl")]
     fn test_role_add_delete() -> Result<(), Error> {
         setup_acl_tree_config();
 
