@@ -2,10 +2,13 @@
 
 pub mod types;
 
-#[cfg(feature = "impl")]
+#[cfg(feature = "acl")]
 pub mod acl;
 
-#[cfg(feature = "impl")]
+#[cfg(feature = "api")]
+pub mod api;
+
+#[cfg(feature = "acl")]
 pub mod init;
 
 #[cfg(feature = "impl")]
