@@ -245,6 +245,12 @@ pub fn handle_method(mut attribs: JSONObject, func: syn::ItemFn) -> Result<Token
         .transpose()?
         .unwrap_or(false);
 
+    let unstable: bool = attribs
+        .remove("unstable")
+        .map(TryFrom::try_from)
+        .transpose()?
+        .unwrap_or(false);
+
     if !attribs.is_empty() {
         error!(
             attribs.span(),
@@ -329,7 +335,8 @@ pub fn handle_method(mut attribs: JSONObject, func: syn::ItemFn) -> Result<Token
             #returns_schema_setter
             #access_setter
             .reload_timezone(#reload_timezone)
-            .protected(#protected);
+            .protected(#protected)
+            .unstable(#unstable);
 
         #default_consts
 
@@ -751,7 +758,7 @@ fn extract_normal_parameter(
             if let Some(def) = &default_value {
                 let name_uc = name.as_ident().to_string().to_uppercase();
                 let name = Ident::new(
-                    &format!("API_METHOD_{}_PARAM_DEFAULT_{}", func_uc, name_uc),
+                    &format!("API_METHOD_{func_uc}_PARAM_DEFAULT_{name_uc}"),
                     span,
                 );
 
