@@ -11,7 +11,7 @@ use std::time::Duration;
 
 use handlebars::{
     Context as HandlebarsContext, Handlebars, Helper, HelperResult, Output, RenderContext,
-    RenderError as HandlebarsRenderError,
+    RenderErrorReason,
 };
 use http::Request;
 use percent_encoding::AsciiSet;
@@ -200,7 +200,7 @@ pub struct KeyAndBase64Val {
 impl KeyAndBase64Val {
     #[cfg(test)]
     pub fn new_with_plain_value(name: &str, value: &str) -> Self {
-        let value = base64::encode(value);
+        let value = proxmox_base64::encode(value);
 
         Self {
             name: name.into(),
@@ -214,7 +214,7 @@ impl KeyAndBase64Val {
     /// text.
     pub fn decode_value(&self) -> Result<String, Error> {
         let value = self.value.as_deref().unwrap_or_default();
-        let bytes = base64::decode(value).map_err(|_| {
+        let bytes = proxmox_base64::decode(value).map_err(|_| {
             Error::Generic(format!(
                 "could not decode base64 value with key '{}'",
                 self.name
@@ -374,7 +374,7 @@ impl WebhookEndpoint {
 
     fn base64_decode(&self, s: &str) -> Result<String, Error> {
         // Also here, TODO: revisit Error variants for the *whole* crate.
-        let s = base64::decode(s)
+        let s = proxmox_base64::decode(s)
             .map_err(|err| Error::Generic(format!("could not decode base64 value: {err}")))?;
 
         String::from_utf8(s).map_err(|err| {
@@ -430,7 +430,7 @@ fn handlebars_percent_encode(
     let param0 = h
         .param(0)
         .and_then(|v| v.value().as_str())
-        .ok_or_else(|| HandlebarsRenderError::new("url-encode: missing parameter"))?;
+        .ok_or_else(|| RenderErrorReason::ParamNotFoundForIndex("url-encode", 0))?;
 
     // See https://developer.mozilla.org/en-US/docs/Glossary/Percent-encoding
     const FRAGMENT: &AsciiSet = &percent_encoding::CONTROLS
@@ -471,9 +471,10 @@ fn handlebars_json(
     let param0 = h
         .param(0)
         .map(|v| v.value())
-        .ok_or_else(|| HandlebarsRenderError::new("json: missing parameter"))?;
+        .ok_or_else(|| RenderErrorReason::ParamNotFoundForIndex("json", 0))?;
 
-    let json = serde_json::to_string(param0)?;
+    let json =
+        serde_json::to_string(param0).map_err(|err| RenderErrorReason::NestedError(err.into()))?;
     out.write(&json)?;
 
     Ok(())
@@ -489,10 +490,11 @@ fn handlebars_escape(
     let text = h
         .param(0)
         .and_then(|v| v.value().as_str())
-        .ok_or_else(|| HandlebarsRenderError::new("escape: missing text parameter"))?;
+        .ok_or_else(|| RenderErrorReason::ParamNotFoundForIndex("escape", 0))?;
 
     let val = Value::String(text.to_string());
-    let json = serde_json::to_string(&val)?;
+    let json =
+        serde_json::to_string(&val).map_err(|err| RenderErrorReason::NestedError(err.into()))?;
     out.write(&json[1..json.len() - 1])?;
 
     Ok(())
@@ -547,7 +549,7 @@ hello%20world
                 header: vec![
                     KeyAndBase64Val::new_with_plain_value("X-Severity", "{{ severity }}").into(),
                 ],
-                body: Some(base64::encode(body_template)),
+                body: Some(proxmox_base64::encode(body_template)),
                 ..Default::default()
             },
             private_config: WebhookPrivateConfig {
