@@ -433,7 +433,7 @@ impl CommandLine {
         self
     }
 
-    pub fn parse<A>(&self, rpcenv: &mut CliEnvironment, args: A) -> Result<Invocation, Error>
+    pub fn parse<A>(&self, rpcenv: &mut CliEnvironment, args: A) -> Result<Invocation<'_>, Error>
     where
         A: IntoIterator<Item = String>,
     {
@@ -520,7 +520,7 @@ impl<'cli> CommandLineParseState<'cli> {
             cmds.sort();
             let list = cmds.join(", ");
 
-            let err_msg = format!("no command specified.\nPossible commands: {}", list);
+            let err_msg = format!("no command specified.\nPossible commands: {list}");
             print_nested_usage_error(&self.prefix, cli, &err_msg);
             return Err(format_err!("{}", err_msg));
         }
