@@ -57,7 +57,7 @@ impl Error {
     }
 
     fn invalid<T: fmt::Display>(msg: T) -> Self {
-        Self::msg(format!("schema validation failed: {}", msg))
+        Self::msg(format!("schema validation failed: {msg}"))
     }
 }
 
@@ -173,17 +173,13 @@ impl<'de> de::Deserializer<'de> for SchemaDeserializer<'de, '_> {
                     .map_err(|_| Error::msg(format!("not a boolean: {:?}", self.input)))?,
             ),
             Schema::Integer(schema) => {
-                // FIXME: isize vs explicit i64, needs fixing in schema check_constraints api
-                let value: isize = self
+                let value: i64 = self
                     .input
                     .parse()
                     .map_err(|_| Error::msg(format!("not an integer: {:?}", self.input)))?;
 
                 schema.check_constraints(value).map_err(Error::invalid)?;
 
-                let value: i64 = i64::try_from(value)
-                    .map_err(|_| Error::invalid("isize did not fit into i64"))?;
-
                 if let Ok(value) = u64::try_from(value) {
                     visitor.visit_u64(value)
                 } else {
@@ -273,13 +269,11 @@ impl<'de> de::Deserializer<'de> for SchemaDeserializer<'de, '_> {
                     self.deserialize_property_string(visitor, schema)
                 }
                 _ => Err(Error::msg(format!(
-                    "cannot deserialize struct '{}' with a string schema",
-                    name
+                    "cannot deserialize struct '{name}' with a string schema"
                 ))),
             },
             _ => Err(Error::msg(format!(
-                "cannot deserialize struct '{}' with non-object schema",
-                name,
+                "cannot deserialize struct '{name}' with non-object schema",
             ))),
         }
     }
@@ -332,8 +326,7 @@ impl<'de> de::Deserializer<'de> for SchemaDeserializer<'de, '_> {
         match self.schema {
             Schema::String(_) => visitor.visit_enum(self.input.into_deserializer()),
             _ => Err(Error::msg(format!(
-                "cannot deserialize enum '{}' with non-string schema",
-                name,
+                "cannot deserialize enum '{name}' with non-string schema",
             ))),
         }
     }
