@@ -83,7 +83,7 @@ impl ParameterError {
                 self.extend(
                     param_err
                         .into_iter()
-                        .map(|(key, err)| (format!("{}/{}", prefix, key), err)),
+                        .map(|(key, err)| (format!("{prefix}/{key}"), err)),
                 );
             }
             Err(err) => self.push(prefix.to_string(), err),
@@ -108,7 +108,7 @@ impl fmt::Display for ParameterError {
             } else {
                 msg.push_str("parameter verification failed:\n");
                 for (name, err) in self.error_list.iter() {
-                    let _ = writeln!(msg, "- '{}': {}", name, err);
+                    let _ = writeln!(msg, "- '{name}': {err}");
                 }
             }
         }
@@ -228,11 +228,11 @@ impl BooleanSchema {
 pub struct IntegerSchema {
     pub description: &'static str,
     /// Optional minimum.
-    pub minimum: Option<isize>,
+    pub minimum: Option<i64>,
     /// Optional maximum.
-    pub maximum: Option<isize>,
+    pub maximum: Option<i64>,
     /// Optional default.
-    pub default: Option<isize>,
+    pub default: Option<i64>,
 }
 
 impl IntegerSchema {
@@ -250,17 +250,17 @@ impl IntegerSchema {
         self
     }
 
-    pub const fn default(mut self, default: isize) -> Self {
+    pub const fn default(mut self, default: i64) -> Self {
         self.default = Some(default);
         self
     }
 
-    pub const fn minimum(mut self, minimum: isize) -> Self {
+    pub const fn minimum(mut self, minimum: i64) -> Self {
         self.minimum = Some(minimum);
         self
     }
 
-    pub const fn maximum(mut self, maximum: isize) -> Self {
+    pub const fn maximum(mut self, maximum: i64) -> Self {
         self.maximum = Some(maximum);
         self
     }
@@ -269,7 +269,7 @@ impl IntegerSchema {
         Schema::Integer(self)
     }
 
-    pub fn check_constraints(&self, value: isize) -> Result<(), Error> {
+    pub fn check_constraints(&self, value: i64) -> Result<(), Error> {
         if let Some(minimum) = self.minimum {
             if value < minimum {
                 bail!(
@@ -296,7 +296,7 @@ impl IntegerSchema {
     /// Verify JSON value using an `IntegerSchema`.
     pub fn verify_json(&self, data: &Value) -> Result<(), Error> {
         if let Some(value) = data.as_i64() {
-            self.check_constraints(value as isize)
+            self.check_constraints(value)
         } else {
             bail!("Expected integer value.");
         }
@@ -419,6 +419,9 @@ pub struct StringSchema {
     pub max_length: Option<usize>,
     /// Optional microformat.
     pub format: Option<&'static ApiStringFormat>,
+    /// Declares the `format` to be a hint for documentation purposes, but other values will be
+    /// accepted for backward/forward compatibility purposes.
+    pub format_is_optional: bool,
     /// A text representation of the format/type (used to generate documentation).
     pub type_text: Option<&'static str>,
 }
@@ -431,6 +434,7 @@ impl StringSchema {
             min_length: None,
             max_length: None,
             format: None,
+            format_is_optional: false,
             type_text: None,
         }
     }
@@ -465,6 +469,11 @@ impl StringSchema {
         self
     }
 
+    pub const fn format_is_optional(mut self, is_optional: bool) -> Self {
+        self.format_is_optional = is_optional;
+        self
+    }
+
     pub const fn schema(self) -> Schema {
         Schema::String(self)
     }
@@ -488,6 +497,10 @@ impl StringSchema {
     pub fn check_constraints(&self, value: &str) -> Result<(), Error> {
         self.check_length(value.chars().count())?;
 
+        if self.format_is_optional {
+            return Ok(());
+        }
+
         if let Some(ref format) = self.format {
             match format {
                 ApiStringFormat::Pattern(regex) => {
@@ -1406,7 +1419,7 @@ impl Schema {
                 Value::Bool(res)
             }
             Schema::Integer(integer_schema) => {
-                let res: isize = value_str.parse()?;
+                let res: i64 = value_str.parse()?;
                 integer_schema.check_constraints(res)?;
                 Value::Number(res.into())
             }
@@ -1441,7 +1454,7 @@ impl Schema {
                         if let Some(key) = default_key {
                             param_list.push((key.to_string(), value.into_owned()));
                         } else {
-                            bail!("Value without key, but schema does not define a default key.");
+                            bail!("Value '{value}' without key, but schema does not define a default key.");
                         }
                     }
                 }
@@ -1819,10 +1832,10 @@ impl ApiStringFormat {
 impl std::fmt::Debug for ApiStringFormat {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            ApiStringFormat::VerifyFn(fnptr) => write!(f, "VerifyFn({:p}", fnptr),
-            ApiStringFormat::Enum(variants) => write!(f, "Enum({:?}", variants),
-            ApiStringFormat::Pattern(regex) => write!(f, "Pattern({:?}", regex),
-            ApiStringFormat::PropertyString(schema) => write!(f, "PropertyString({:?}", schema),
+            ApiStringFormat::VerifyFn(fnptr) => write!(f, "VerifyFn({fnptr:p}"),
+            ApiStringFormat::Enum(variants) => write!(f, "Enum({variants:?}"),
+            ApiStringFormat::Pattern(regex) => write!(f, "Pattern({regex:?}"),
+            ApiStringFormat::PropertyString(schema) => write!(f, "PropertyString({schema:?}"),
         }
     }
 }
