@@ -41,6 +41,29 @@ const RFC5987SET: &AsciiSet = &CONTROLS
     .add(b'{')
     .add(b'}');
 
+/// base64 encode and hard-wrap the base64 encoded string every 72 characters. this improves
+/// compatibility.
+fn encode_base64_formatted<T: AsRef<[u8]>>(raw: T) -> String {
+    const TEXT_WIDTH: usize = 72;
+
+    let encoded = proxmox_base64::encode(raw);
+    let bytes = encoded.as_bytes();
+
+    let lines = bytes.len().div_ceil(TEXT_WIDTH);
+    let mut out = Vec::with_capacity(bytes.len() + lines - 1); // account for 1 newline per line
+
+    for (line, chunk) in bytes.chunks(TEXT_WIDTH).enumerate() {
+        out.extend_from_slice(chunk);
+        if line + 1 != lines {
+            // do not end last line with newline to give caller control over doing so.
+            out.push(b'\n');
+        }
+    }
+
+    // SAFETY: base64 encoded, which is 7-bit chars (ASCII) and thus always valid UTF8
+    unsafe { String::from_utf8_unchecked(out) }
+}
+
 struct Recipient {
     name: Option<String>,
     email: String,
@@ -55,7 +78,11 @@ impl Recipient {
     fn format_recipient(&self) -> String {
         if let Some(name) = &self.name {
             if !name.is_ascii() {
-                format!("=?utf-8?B?{}?= <{}>", base64::encode(name), self.email)
+                format!(
+                    "=?utf-8?B?{}?= <{}>",
+                    proxmox_base64::encode(name),
+                    self.email
+                )
             } else {
                 format!("{name} <{}>", self.email)
             }
@@ -77,39 +104,28 @@ impl Attachment<'_> {
 
         let mut attachment = String::new();
 
+        let encoded_filename = if self.filename.is_ascii() {
+            &self.filename
+        } else {
+            &format!("=?utf-8?B?{}?=", proxmox_base64::encode(&self.filename))
+        };
+
         let _ = writeln!(attachment, "\n--{file_boundary}");
         let _ = writeln!(
             attachment,
-            "Content-Type: {}; name=\"{}\"",
-            self.mime, self.filename
+            "Content-Type: {}; name=\"{encoded_filename}\"",
+            self.mime,
         );
 
-        // both `filename` and `filename*` are included for additional compatability
+        // both `filename` and `filename*` are included for additional compatibility
         let _ = writeln!(
             attachment,
-            "Content-Disposition: attachment; filename=\"{}\"; filename*=UTF-8''{}",
-            self.filename,
+            "Content-Disposition: attachment; filename=\"{encoded_filename}\";\n\tfilename*=UTF-8''{}",
             utf8_percent_encode(&self.filename, RFC5987SET)
         );
-        attachment.push_str("Content-Transfer-Encoding: base64\n\n");
 
-        // base64 encode the attachment and hard-wrap the base64 encoded string every 72
-        // characters. this improves compatability.
-        attachment.push_str(
-            &base64::encode(self.content)
-                .chars()
-                .enumerate()
-                .flat_map(|(i, c)| {
-                    if i != 0 && i % 72 == 0 {
-                        Some('\n')
-                    } else {
-                        None
-                    }
-                    .into_iter()
-                    .chain(std::iter::once(c))
-                })
-                .collect::<String>(),
-        );
+        attachment.push_str("Content-Transfer-Encoding: base64\n\n");
+        attachment.push_str(&encode_base64_formatted(self.content));
 
         attachment
     }
@@ -125,6 +141,7 @@ pub struct Mail<'a> {
     body_html: Option<String>,
     attachments: Vec<Attachment<'a>>,
     mask_participants: bool,
+    noreply: Option<Recipient>,
 }
 
 impl<'a> Mail<'a> {
@@ -142,6 +159,7 @@ impl<'a> Mail<'a> {
             body_html: None,
             attachments: Vec::new(),
             mask_participants: true,
+            noreply: None,
         }
     }
 
@@ -212,7 +230,7 @@ impl<'a> Mail<'a> {
     /// Builder-style method to conveniently add an attachment with a specific filename and
     /// mime-type to an email.
     ///
-    /// Note: Adding attachements triggers `multipart/mixed` mode.
+    /// Note: Adding attachments triggers `multipart/mixed` mode.
     pub fn with_attachment(mut self, filename: &str, mime_type: &str, content: &'a [u8]) -> Self {
         self.add_attachment(filename, mime_type, content);
         self
@@ -254,6 +272,22 @@ impl<'a> Mail<'a> {
         self
     }
 
+    /// Set the receiver that is used when the mail is send in masked mode. `Undisclosed <noreply>`
+    /// by default.
+    pub fn set_masked_mail_and_name(&mut self, name: &str, email: &str) {
+        self.noreply = Some(Recipient {
+            email: email.to_owned(),
+            name: Some(name.to_owned()),
+        });
+    }
+
+    /// Builder-style method to set the receiver when the mail is send in masked mode. `Undisclosed
+    /// <noreply>` by default.
+    pub fn with_masked_receiver(mut self, name: &str, email: &str) -> Self {
+        self.set_masked_mail_and_name(name, email);
+        self
+    }
+
     /// Sends the email. This will fail if no recipients have been added.
     ///
     /// Note: An `Auto-Submitted: auto-generated` header is added to avoid triggering OOO and
@@ -391,7 +425,11 @@ impl<'a> Mail<'a> {
             header.push_str("Content-Type: multipart/alternative;\n");
             writeln!(header, "\tboundary=\"{html_boundary}\"")?;
             header.push_str("MIME-Version: 1.0\n");
-        } else if !self.subject.is_ascii() || !self.mail_author.is_ascii() || encoded_to {
+        } else if !self.subject.is_ascii()
+            || !self.mail_author.is_ascii()
+            || !self.body_txt.is_ascii()
+            || encoded_to
+        {
             header.push_str("MIME-Version: 1.0\n");
         }
 
@@ -399,7 +437,7 @@ impl<'a> Mail<'a> {
             writeln!(
                 header,
                 "Subject: =?utf-8?B?{}?=",
-                base64::encode(&self.subject)
+                proxmox_base64::encode(&self.subject)
             )?;
         } else {
             writeln!(header, "Subject: {}", self.subject)?;
@@ -409,7 +447,7 @@ impl<'a> Mail<'a> {
             writeln!(
                 header,
                 "From: =?utf-8?B?{}?= <{}>",
-                base64::encode(&self.mail_author),
+                proxmox_base64::encode(&self.mail_author),
                 self.mail_from
             )?;
         } else {
@@ -418,12 +456,16 @@ impl<'a> Mail<'a> {
 
         let to = if self.to.len() > 1 && self.mask_participants {
             // don't disclose all recipients if the mail goes out to multiple
-            let recipient = Recipient {
-                name: Some("Undisclosed".to_string()),
-                email: "noreply".to_string(),
-            };
-
-            recipient.format_recipient()
+            self.noreply
+                .as_ref()
+                .map(|f| f.format_recipient())
+                .unwrap_or_else(|| {
+                    Recipient {
+                        name: Some("Undisclosed".to_string()),
+                        email: "noreply".to_string(),
+                    }
+                    .format_recipient()
+                })
         } else {
             self.to
                 .iter()
@@ -466,15 +508,28 @@ impl<'a> Mail<'a> {
 
         body.push_str("Content-Type: text/plain;\n");
         body.push_str("\tcharset=\"UTF-8\"\n");
-        body.push_str("Content-Transfer-Encoding: 8bit\n\n");
-        body.push_str(&self.body_txt);
+
+        if self.body_txt.is_ascii() {
+            body.push_str("Content-Transfer-Encoding: 7bit\n\n");
+            body.push_str(&self.body_txt);
+        } else {
+            body.push_str("Content-Transfer-Encoding: base64\n\n");
+            body.push_str(&encode_base64_formatted(&self.body_txt));
+        }
 
         if let Some(html) = &self.body_html {
             writeln!(body, "\n--{html_boundary}")?;
             body.push_str("Content-Type: text/html;\n");
             body.push_str("\tcharset=\"UTF-8\"\n");
-            body.push_str("Content-Transfer-Encoding: 8bit\n\n");
-            body.push_str(html);
+
+            if html.is_ascii() {
+                body.push_str("Content-Transfer-Encoding: 7bit\n\n");
+                body.push_str(html);
+            } else {
+                body.push_str("Content-Transfer-Encoding: base64\n\n");
+                body.push_str(&encode_base64_formatted(html));
+            }
+
             write!(body, "\n--{html_boundary}--")?;
         }
 
@@ -486,6 +541,25 @@ impl<'a> Mail<'a> {
 mod test {
     use super::*;
 
+    /// Compare two multi-line strings, ignoring any line that starts with 'Date:'.
+    ///
+    /// The `Date` header is formatted in the local timezone, which means our
+    /// tests are sensitive to the timezone of the machine running the tests.
+    /// Simplest solution is to just ignore the date header.
+    #[track_caller]
+    fn assert_lines_equal_ignore_date(s1: &str, s2: &str) {
+        let lines1 = s1.lines();
+        let lines2 = s2.lines();
+
+        for ((i, line1), line2) in lines1.enumerate().zip(lines2) {
+            if !(line1.starts_with("Date:") && line2.starts_with("Date:")) {
+                assert_eq!(line1, line2, "lines at {i} do not match");
+            }
+        }
+
+        assert_eq!(s1.lines().count(), s2.lines().count());
+    }
+
     #[test]
     fn email_without_recipients_fails() {
         let result = Mail::new("Sender", "mail@example.com", "hi", "body").send();
@@ -511,8 +585,8 @@ mod test {
 
         let body = mail.format_mail(0).expect("could not format mail");
 
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"Subject: Subject Line
 From: Sender Name <mailfrom@example.com>
 To: Receiver Name <receiver@example.com>
@@ -520,10 +594,10 @@ Date: Thu, 01 Jan 1970 01:00:00 +0100
 Auto-Submitted: auto-generated;
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 This is just ascii text.
-Nothing too special."#
+Nothing too special."#,
         )
     }
 
@@ -541,8 +615,8 @@ Nothing too special."#
 
         let body = mail.format_mail(0).expect("could not format mail");
 
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"Subject: Subject Line
 From: Sender Name <mailfrom@example.com>
 To: Undisclosed <noreply>
@@ -550,10 +624,10 @@ Date: Thu, 01 Jan 1970 01:00:00 +0100
 Auto-Submitted: auto-generated;
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 This is just ascii text.
-Nothing too special."#
+Nothing too special."#,
         )
     }
 
@@ -572,8 +646,8 @@ Nothing too special."#
 
         let body = mail.format_mail(0).expect("could not format mail");
 
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"MIME-Version: 1.0
 Subject: Subject Line
 From: Sender Name <mailfrom@example.com>
@@ -582,10 +656,41 @@ Date: Thu, 01 Jan 1970 01:00:00 +0100
 Auto-Submitted: auto-generated;
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 This is just ascii text.
-Nothing too special."#
+Nothing too special."#,
+        )
+    }
+
+    #[test]
+    fn multiple_receiver_custom_masked() {
+        let mail = Mail::new(
+            "Sender Name",
+            "mailfrom@example.com",
+            "Subject Line",
+            "This is just ascii text.\nNothing too special.",
+        )
+        .with_recipient_and_name("Receiver Name", "receiver@example.com")
+        .with_recipient("two@example.com")
+        .with_recipient_and_name("mäx müstermänn", "mm@example.com")
+        .with_masked_receiver("Example Receiver", "noanswer@example.com");
+
+        let body = mail.format_mail(0).expect("could not format mail");
+
+        assert_lines_equal_ignore_date(
+            &body,
+            r#"Subject: Subject Line
+From: Sender Name <mailfrom@example.com>
+To: Example Receiver <noanswer@example.com>
+Date: Thu, 01 Jan 1970 01:00:00 +0100
+Auto-Submitted: auto-generated;
+Content-Type: text/plain;
+	charset="UTF-8"
+Content-Transfer-Encoding: 7bit
+
+This is just ascii text.
+Nothing too special."#,
         )
     }
 
@@ -601,8 +706,8 @@ Nothing too special."#
 
         let body = mail.format_mail(1732806251).expect("could not format mail");
 
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"MIME-Version: 1.0
 Subject: =?utf-8?B?U3ViamVjdCBMaW5lIPCfp5E=?=
 From: =?utf-8?B?VVRGLTggU2VuZGVyIE5hbWUg8J+Tpw==?= <differentfrom@example.com>
@@ -611,12 +716,10 @@ Date: Thu, 28 Nov 2024 16:04:11 +0100
 Auto-Submitted: auto-generated;
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: base64
 
-This utf-8 email should handle emojis
-🧑📧
-and weird german characters: öäüß
-and more."#
+VGhpcyB1dGYtOCBlbWFpbCBzaG91bGQgaGFuZGxlIGVtb2ppcwrwn6eR8J+TpwphbmQgd2Vp
+cmQgZ2VybWFuIGNoYXJhY3RlcnM6IMO2w6TDvMOfCmFuZCBtb3JlLg=="#,
         )
     }
 
@@ -631,8 +734,8 @@ and more."#
         .with_recipient("receiver@example.com")
         .with_html_alt("<html lang=\"de-at\"><head></head><body>\n\t<pre>\n\t\tLorem Ipsum Dolor Sit Amet\n\t</pre>\n</body></html>");
         let body = mail.format_mail(1732806251).expect("could not format mail");
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"Content-Type: multipart/alternative;
 	boundary="----_=_NextPart_002_1732806251"
 MIME-Version: 1.0
@@ -647,21 +750,21 @@ This is a multi-part message in MIME format.
 ------_=_NextPart_002_1732806251
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 Lorem Ipsum Dolor Sit
 Amet
 ------_=_NextPart_002_1732806251
 Content-Type: text/html;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 <html lang="de-at"><head></head><body>
 	<pre>
 		Lorem Ipsum Dolor Sit Amet
 	</pre>
 </body></html>
-------_=_NextPart_002_1732806251--"#
+------_=_NextPart_002_1732806251--"#,
         )
     }
 
@@ -685,8 +788,8 @@ Content-Transfer-Encoding: 8bit
         .with_attachment("deadbeef.bin", "application/octet-stream", &bin);
 
         let body = mail.format_mail(1732806251).expect("could not format mail");
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"Content-Type: multipart/mixed;
 	boundary="----_=_NextPart_001_1732806251"
 MIME-Version: 1.0
@@ -701,18 +804,19 @@ This is a multi-part message in MIME format.
 ------_=_NextPart_001_1732806251
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 Lorem Ipsum Dolor Sit
 Amet
 ------_=_NextPart_001_1732806251
 Content-Type: application/octet-stream; name="deadbeef.bin"
-Content-Disposition: attachment; filename="deadbeef.bin"; filename*=UTF-8''deadbeef.bin
+Content-Disposition: attachment; filename="deadbeef.bin";
+	filename*=UTF-8''deadbeef.bin
 Content-Transfer-Encoding: base64
 
 3q2+796tvu/erb7v3q3erb7v3q2+796tvu/erd6tvu/erb7v3q2+796t3q2+796tvu/erb7v
 3q2+796tvu8=
-------_=_NextPart_001_1732806251--"#
+------_=_NextPart_001_1732806251--"#,
         )
     }
 
@@ -739,8 +843,8 @@ Content-Transfer-Encoding: base64
 
         let body = mail.format_mail(1732806251).expect("could not format mail");
 
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"Content-Type: multipart/mixed;
 	boundary="----_=_NextPart_001_1732806251"
 MIME-Version: 1.0
@@ -759,14 +863,14 @@ MIME-Version: 1.0
 ------_=_NextPart_002_1732806251
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 Lorem Ipsum Dolor Sit
 Amet
 ------_=_NextPart_002_1732806251
 Content-Type: text/html;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 <html lang="de-at"><head></head><body>
 	<pre>
@@ -776,19 +880,21 @@ Content-Transfer-Encoding: 8bit
 ------_=_NextPart_002_1732806251--
 ------_=_NextPart_001_1732806251
 Content-Type: application/octet-stream; name="deadbeef.bin"
-Content-Disposition: attachment; filename="deadbeef.bin"; filename*=UTF-8''deadbeef.bin
+Content-Disposition: attachment; filename="deadbeef.bin";
+	filename*=UTF-8''deadbeef.bin
 Content-Transfer-Encoding: base64
 
 3q2+796tvu/erb7v3q3erb7v3q2+796tvu/erd6tvu/erb7v3q2+796t3q2+796tvu/erb7v
 3q2+796tvu8=
 ------_=_NextPart_001_1732806251
-Content-Type: image/bmp; name="🐄💀.bin"
-Content-Disposition: attachment; filename="🐄💀.bin"; filename*=UTF-8''%F0%9F%90%84%F0%9F%92%80.bin
+Content-Type: image/bmp; name="=?utf-8?B?8J+QhPCfkoAuYmlu?="
+Content-Disposition: attachment; filename="=?utf-8?B?8J+QhPCfkoAuYmlu?=";
+	filename*=UTF-8''%F0%9F%90%84%F0%9F%92%80.bin
 Content-Transfer-Encoding: base64
 
 3q2+796tvu/erb7v3q3erb7v3q2+796tvu/erd6tvu/erb7v3q2+796t3q2+796tvu/erb7v
 3q2+796tvu8=
-------_=_NextPart_001_1732806251--"#
+------_=_NextPart_001_1732806251--"#,
         )
     }
 
@@ -805,8 +911,8 @@ Content-Transfer-Encoding: base64
 
         let body = mail.format_mail(1718977850).expect("could not format mail");
 
-        assert_eq!(
-            body,
+        assert_lines_equal_ignore_date(
+            &body,
             r#"Content-Type: multipart/alternative;
 	boundary="----_=_NextPart_002_1718977850"
 MIME-Version: 1.0
@@ -821,16 +927,92 @@ This is a multi-part message in MIME format.
 ------_=_NextPart_002_1718977850
 Content-Type: text/plain;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 This is the plain body
 ------_=_NextPart_002_1718977850
 Content-Type: text/html;
 	charset="UTF-8"
-Content-Transfer-Encoding: 8bit
+Content-Transfer-Encoding: 7bit
 
 <body>This is the HTML body</body>
-------_=_NextPart_002_1718977850--"#
+------_=_NextPart_002_1718977850--"#,
         );
     }
+
+    #[test]
+    fn multipart_plain_text_html_alternative_attachments_ascii_compat() {
+        let bin: [u8; 62] = [
+            0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad,
+            0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad,
+            0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad,
+            0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef, 0xde, 0xad,
+            0xbe, 0xef, 0xde, 0xad, 0xbe, 0xef,
+        ];
+
+        let mail = Mail::new(
+            "Sender Näme",
+            "from@example.com",
+            "Subject Linë",
+            "Lorem Ipsum Dolor Sit\nAmët",
+        )
+        .with_recipient_and_name("Receiver Näme", "receiver@example.com")
+        .with_attachment("deadbeef.bin", "application/octet-stream", &bin)
+        .with_attachment("🐄💀.bin", "image/bmp", &bin)
+        .with_html_alt("<html lang=\"de-at\"><head></head><body>\n\t<pre>\n\t\tLorem Ipsum Dölor Sit Amet\n\t</pre>\n</body></html>");
+
+        let body = mail.format_mail(1732806251).expect("could not format mail");
+
+        assert!(body.is_ascii());
+
+        assert_lines_equal_ignore_date(
+            &body,
+            r#"Content-Type: multipart/mixed;
+	boundary="----_=_NextPart_001_1732806251"
+MIME-Version: 1.0
+Subject: =?utf-8?B?U3ViamVjdCBMaW7Dqw==?=
+From: =?utf-8?B?U2VuZGVyIE7DpG1l?= <from@example.com>
+To: =?utf-8?B?UmVjZWl2ZXIgTsOkbWU=?= <receiver@example.com>
+Date: Thu, 28 Nov 2024 16:04:11 +0100
+Auto-Submitted: auto-generated;
+
+This is a multi-part message in MIME format.
+
+------_=_NextPart_001_1732806251
+Content-Type: multipart/alternative; boundary="----_=_NextPart_002_1732806251"
+MIME-Version: 1.0
+
+------_=_NextPart_002_1732806251
+Content-Type: text/plain;
+	charset="UTF-8"
+Content-Transfer-Encoding: base64
+
+TG9yZW0gSXBzdW0gRG9sb3IgU2l0CkFtw6t0
+------_=_NextPart_002_1732806251
+Content-Type: text/html;
+	charset="UTF-8"
+Content-Transfer-Encoding: base64
+
+PGh0bWwgbGFuZz0iZGUtYXQiPjxoZWFkPjwvaGVhZD48Ym9keT4KCTxwcmU+CgkJTG9yZW0g
+SXBzdW0gRMO2bG9yIFNpdCBBbWV0Cgk8L3ByZT4KPC9ib2R5PjwvaHRtbD4=
+------_=_NextPart_002_1732806251--
+------_=_NextPart_001_1732806251
+Content-Type: application/octet-stream; name="deadbeef.bin"
+Content-Disposition: attachment; filename="deadbeef.bin";
+	filename*=UTF-8''deadbeef.bin
+Content-Transfer-Encoding: base64
+
+3q2+796tvu/erb7v3q3erb7v3q2+796tvu/erd6tvu/erb7v3q2+796t3q2+796tvu/erb7v
+3q2+796tvu8=
+------_=_NextPart_001_1732806251
+Content-Type: image/bmp; name="=?utf-8?B?8J+QhPCfkoAuYmlu?="
+Content-Disposition: attachment; filename="=?utf-8?B?8J+QhPCfkoAuYmlu?=";
+	filename*=UTF-8''%F0%9F%90%84%F0%9F%92%80.bin
+Content-Transfer-Encoding: base64
+
+3q2+796tvu/erb7v3q3erb7v3q2+796tvu/erd6tvu/erb7v3q2+796t3q2+796tvu/erb7v
+3q2+796tvu8=
+------_=_NextPart_001_1732806251--"#,
+        )
+    }
 }
