@@ -5,7 +5,7 @@
 use std::ffi::{OsStr, OsString};
 use std::future::Future;
 use std::io;
-use std::mem::{self, size_of, size_of_val, MaybeUninit};
+use std::mem::{self, MaybeUninit, size_of, size_of_val};
 use std::ops::Range;
 use std::os::unix::ffi::{OsStrExt, OsStringExt};
 use std::path::{Path, PathBuf};
@@ -112,7 +112,7 @@ where
 }
 
 /// Allow using trait objects for `T: ReadAt`
-impl<'d> ReadAt for &(dyn ReadAt + 'd) {
+impl ReadAt for &(dyn ReadAt + '_) {
     fn start_read_at<'a>(
         self: Pin<&'a Self>,
         cx: &mut Context,
@@ -733,7 +733,7 @@ impl<T: Clone + ReadAt> DirectoryImpl<T> {
         Ok((file_name, file_ofs + head.full_size()))
     }
 
-    pub fn read_dir(&self) -> ReadDirImpl<T> {
+    pub fn read_dir(&self) -> ReadDirImpl<'_, T> {
         ReadDirImpl::new(self, 0)
     }
 
@@ -873,7 +873,7 @@ pub(crate) struct DirEntryImpl<'a, T: Clone + ReadAt> {
     caches: Arc<Caches>,
 }
 
-impl<'a, T: Clone + ReadAt> DirEntryImpl<'a, T> {
+impl<T: Clone + ReadAt> DirEntryImpl<'_, T> {
     pub fn file_name(&self) -> &Path {
         &self.file_name
     }
@@ -1060,7 +1060,12 @@ impl<T: ReadAt> decoder::SeqRead for SeqReadAtAdapter<T> {
                             io::Result::Ok(ReadResult { len, buffer })
                         });
                     // Ditch the self-reference life-time now:
-                    this.future = Some(unsafe { mem::transmute(future) });
+                    this.future = Some(unsafe {
+                        mem::transmute::<
+                            Pin<Box<dyn Future<Output = io::Result<ReadResult>> + '_>>,
+                            Pin<Box<dyn Future<Output = io::Result<ReadResult>> + 'static>>,
+                        >(future)
+                    });
                 }
                 Some(mut fut) => match fut.as_mut().poll(cx) {
                     Poll::Pending => {
