@@ -7,7 +7,7 @@
 use std::ffi::OsString;
 use std::future::poll_fn;
 use std::io;
-use std::mem::{self, size_of, size_of_val, MaybeUninit};
+use std::mem::{self, MaybeUninit, size_of, size_of_val};
 use std::ops::Range;
 use std::os::unix::ffi::{OsStrExt, OsStringExt};
 use std::path::{Path, PathBuf};
@@ -32,7 +32,11 @@ pub use sync::Decoder;
 static mut SCRATCH_BUFFER: MaybeUninit<[u8; 4096]> = MaybeUninit::uninit();
 
 fn scratch_buffer() -> &'static mut [u8] {
-    unsafe { &mut (*SCRATCH_BUFFER.as_mut_ptr())[..] }
+    unsafe {
+        let ptr: *mut MaybeUninit<[u8; 4096]> = &raw mut SCRATCH_BUFFER;
+        let ptr: &mut [u8; 4096] = &mut *(*ptr).as_mut_ptr();
+        &mut ptr[..]
+    }
 }
 
 /// Sequential read interface used by the decoder's state machine.
@@ -64,7 +68,7 @@ pub trait SeqRead {
 }
 
 /// Allow using trait objects for generics taking a `SeqRead`:
-impl<'a> SeqRead for &mut (dyn SeqRead + 'a) {
+impl SeqRead for &mut (dyn SeqRead + '_) {
     fn poll_seq_read(
         self: Pin<&mut Self>,
         cx: &mut Context,
@@ -371,7 +375,7 @@ impl<I: SeqRead> DecoderImpl<I> {
         }
     }
 
-    pub async fn content_reader(&mut self) -> Result<Option<Contents<I>>, io::Error> {
+    pub async fn content_reader(&mut self) -> Result<Option<Contents<'_, I>>, io::Error> {
         if let State::InPayload {
             offset,
             size,
@@ -844,7 +848,7 @@ impl<'a, T: SeqRead> Contents<'a, T> {
     }
 }
 
-impl<'a, T: SeqRead> SeqRead for Contents<'a, T> {
+impl<T: SeqRead> SeqRead for Contents<'_, T> {
     fn poll_seq_read(
         mut self: Pin<&mut Self>,
         cx: &mut Context,
